"""
队列服务层
处理队列相关的业务逻辑
"""
from typing import Optional, List, Dict, Any
from datetime import datetime, timedelta, timezone
from sqlalchemy import text
from sqlalchemy.ext.asyncio import AsyncSession
import logging
import traceback

logger = logging.getLogger(__name__)


class QueueService:
    """队列服务类"""
    
    @staticmethod
    def get_base_queue_name(queue_name: str) -> str:
        """
        提取基础队列名（去除优先级后缀）
        
        Args:
            queue_name: 完整队列名
            
        Returns:
            基础队列名
        """
        if ':' in queue_name:
            parts = queue_name.rsplit(':', 1)
            if parts[-1].isdigit():
                return parts[0]
        return queue_name
    
    @staticmethod
    async def get_queues_by_namespace(namespace_data_access, namespace: str) -> Dict[str, Any]:
        """
        获取指定命名空间的队列列表
        
        Args:
            namespace_data_access: 命名空间数据访问实例
            namespace: 命名空间
            
        Returns:
            队列列表
        """
        queues_data = await namespace_data_access.get_queue_stats(namespace)
        return {
            "success": True,
            "data": list(set([QueueService.get_base_queue_name(q['queue_name']) for q in queues_data]))
        }
    
    @staticmethod
    async def get_queue_flow_rates(data_access, query) -> Dict[str, Any]:
        """
        获取单个队列的流量速率（入队、开始执行、完成）
        
        Args:
            data_access: 数据访问层实例
            query: 时间范围查询对象
            
        Returns:
            队列流量速率数据
        """
        # 处理时间范围
        now = datetime.now(timezone.utc)
        
        if query.start_time and query.end_time:
            # 使用提供的时间范围
            start_time = query.start_time
            end_time = query.end_time
            logger.info(f"使用自定义时间范围: {start_time} 到 {end_time}")
        else:
            # 根据time_range参数计算时间范围
            time_range_map = {
                "15m": timedelta(minutes=15),
                "30m": timedelta(minutes=30),
                "1h": timedelta(hours=1),
                "3h": timedelta(hours=3),
                "6h": timedelta(hours=6),
                "12h": timedelta(hours=12),
                "24h": timedelta(hours=24),
                "7d": timedelta(days=7),
                "30d": timedelta(days=30),
            }
            
            # 优先使用 time_range，如果没有则使用 interval
            time_range_value = query.time_range if query.time_range else query.interval
            delta = time_range_map.get(time_range_value, timedelta(minutes=15))
            
            # 获取队列的最新任务时间，确保图表包含最新数据
            queue_name = query.queues[0] if query.queues else None
            if queue_name:
                latest_time = await data_access.get_latest_task_time(queue_name)
                if latest_time:
                    # 使用最新任务时间作为结束时间
                    end_time = latest_time.replace(second=59, microsecond=999999)  # 包含整分钟
                    logger.info(f"使用最新任务时间: {latest_time}")
                else:
                    # 如果没有任务，使用当前时间
                    end_time = now.replace(second=0, microsecond=0)
            else:
                end_time = now.replace(second=0, microsecond=0)
            
            start_time = end_time - delta
            logger.info(f"使用预设时间范围 {time_range_value}: {start_time} 到 {end_time}, delta: {delta}")
        
        # 确保有队列名称
        if not query.queues or len(query.queues) == 0:
            return {"data": [], "granularity": "minute"}
        
        # 获取第一个队列的流量速率
        queue_name = query.queues[0]
        # TimeRangeQuery 没有 filters 属性，传递 None 或空字典
        filters = getattr(query, 'filters', None)
        data, granularity = await data_access.fetch_queue_flow_rates(
            queue_name, start_time, end_time, filters
        )
        
        return {"data": data, "granularity": granularity}
    
    @staticmethod
    async def get_global_stats(data_access) -> Dict[str, Any]:
        """
        获取全局统计信息
        
        Args:
            data_access: 数据访问层实例
            
        Returns:
            全局统计数据
        """
        stats_data = await data_access.fetch_global_stats()
        return {
            "success": True,
            "data": stats_data
        }
    
    @staticmethod
    async def get_queues_detail(data_access) -> Dict[str, Any]:
        """
        获取队列详细信息
        
        Args:
            data_access: 数据访问层实例
            
        Returns:
            队列详细数据
        """
        queues_data = await data_access.fetch_queues_data()
        return {
            "success": True,
            "data": queues_data
        }
    
    @staticmethod
    async def delete_queue(queue_name: str) -> Dict[str, Any]:
        """
        删除队列
        
        Args:
            queue_name: 队列名称
            
        Returns:
            操作结果
        """
        # TODO: 实现删除队列的逻辑
        logger.info(f"删除队列请求: {queue_name}")
        return {
            "success": True,
            "message": f"队列 {queue_name} 已删除"
        }
    
    @staticmethod
    async def trim_queue(queue_name: str, max_length: int) -> Dict[str, Any]:
        """
        裁剪队列到指定长度
        
        Args:
            queue_name: 队列名称
            max_length: 最大长度
            
        Returns:
            操作结果
        """
        # TODO: 实现裁剪队列的逻辑
        logger.info(f"裁剪队列请求: {queue_name}, 保留 {max_length} 条消息")
        return {
            "success": True,
            "message": f"队列 {queue_name} 已裁剪至 {max_length} 条消息"
        }
    
    @staticmethod
    async def get_queue_stats_v2(
        namespace_data_access,
        namespace: str,
        queue: Optional[str] = None,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        获取队列统计信息v2 - 支持消费者组详情和优先级队列
        
        Args:
            namespace_data_access: 命名空间数据访问实例
            namespace: 命名空间
            queue: 可选，筛选特定队列
            start_time: 开始时间
            end_time: 结束时间
            time_range: 时间范围
            
        Returns:
            队列统计数据
        """
        # 获取命名空间连接
        conn = await namespace_data_access.manager.get_connection(namespace)
        
        # 获取Redis客户端
        redis_client = await conn.get_redis_client(decode=False)
        
        # 获取PostgreSQL会话（可选）
        pg_session = None
        if conn.AsyncSessionLocal:
            pg_session = conn.AsyncSessionLocal()
        
        try:
            # 导入 QueueStatsV2
            from jettask.webui.services.queue_stats_v2 import QueueStatsV2
            
            # 创建统计服务实例
            stats_service = QueueStatsV2(
                redis_client=redis_client,
                pg_session=pg_session,
                redis_prefix=conn.redis_prefix
            )
            
            # 处理时间筛选参数
            time_filter = None
            if time_range or start_time or end_time:
                time_filter = {}
                
                # 如果提供了time_range，计算开始和结束时间
                if time_range and time_range != 'custom':
                    now = datetime.now(timezone.utc)
                    if time_range.endswith('m'):
                        minutes = int(time_range[:-1])
                        time_filter['start_time'] = now - timedelta(minutes=minutes)
                        time_filter['end_time'] = now
                    elif time_range.endswith('h'):
                        hours = int(time_range[:-1])
                        time_filter['start_time'] = now - timedelta(hours=hours)
                        time_filter['end_time'] = now
                    elif time_range.endswith('d'):
                        days = int(time_range[:-1])
                        time_filter['start_time'] = now - timedelta(days=days)
                        time_filter['end_time'] = now
                else:
                    # 使用提供的start_time和end_time
                    if start_time:
                        time_filter['start_time'] = start_time
                    if end_time:
                        time_filter['end_time'] = end_time
            
            # 获取队列统计（使用分组格式）
            stats = await stats_service.get_queue_stats_grouped(time_filter)
            
            # 如果指定了队列筛选，则过滤结果
            if queue:
                stats = [s for s in stats if s['queue_name'] == queue]
            
            return {
                "success": True,
                "data": stats
            }
            
        finally:
            if pg_session:
                await pg_session.close()
            await redis_client.aclose()
    
    @staticmethod
    async def get_tasks_v2(namespace_data_access, namespace: str, body: Dict[str, Any]) -> Dict[str, Any]:
        """
        获取任务列表v2 - 支持tasks和task_runs表连表查询
        
        Args:
            namespace_data_access: 命名空间数据访问实例
            namespace: 命名空间
            body: 请求体参数
            
        Returns:
            任务列表数据
        """
        from sqlalchemy import text
        from datetime import datetime, timezone, timedelta
        
        queue_name = body.get('queue_name')
        page = body.get('page', 1)
        page_size = body.get('page_size', 20)
        filters = body.get('filters', [])
        time_range = body.get('time_range', '1h')
        start_time = body.get('start_time')
        end_time = body.get('end_time')
        sort_field = body.get('sort_field', 'created_at')
        sort_order = body.get('sort_order', 'desc')
        
        if not queue_name:
            raise ValueError("queue_name is required")
        
        # 获取命名空间连接
        conn = await namespace_data_access.manager.get_connection(namespace)
        
        if not conn.pg_config or not conn.async_engine:
            return {
                "success": True,
                "data": [],
                "total": 0
            }
        
        # 解析时间范围
        if start_time and end_time:
            # 使用自定义时间范围
            start_dt = datetime.fromisoformat(start_time.replace('Z', '+00:00'))
            end_dt = datetime.fromisoformat(end_time.replace('Z', '+00:00'))
        else:
            # 使用预定义时间范围
            end_dt = datetime.now(timezone.utc)
            time_deltas = {
                '15m': timedelta(minutes=15),
                '30m': timedelta(minutes=30),
                '1h': timedelta(hours=1),
                '3h': timedelta(hours=3),
                '6h': timedelta(hours=6),
                '12h': timedelta(hours=12),
                '1d': timedelta(days=1),
                '3d': timedelta(days=3),
                '7d': timedelta(days=7),
                '30d': timedelta(days=30)
            }
            delta = time_deltas.get(time_range, timedelta(hours=1))
            start_dt = end_dt - delta
        
        offset = (page - 1) * page_size
        
        async with conn.async_engine.begin() as pg_conn:
            # 构建查询条件
            conditions = [
                "t.namespace = :namespace",
                "t.queue = :queue",
                "t.created_at >= :start_time",
                "t.created_at <= :end_time"
            ]
            query_params = {
                "namespace": namespace,
                "queue": queue_name,
                "start_time": start_dt,
                "end_time": end_dt,
                "limit": page_size,
                "offset": offset
            }
            
            # 处理筛选条件
            for i, filter_item in enumerate(filters):
                field = filter_item.get('field')
                operator = filter_item.get('operator')
                value = filter_item.get('value')
                
                if field and operator and value is not None:
                    param_key = f"filter_{i}"
                    
                    # 映射前端字段到数据库字段（使用payload JSONB列）
                    db_field_map = {
                        'id': 't.stream_id',
                        'task_name': "t.payload::jsonb->'event_data'->>'__task_name'",
                        'status': "t.payload::jsonb->>'status'",
                        'worker_id': "t.payload::jsonb->>'worker_id'",
                        'scheduled_task_id': 't.scheduled_task_id'
                    }
                    
                    db_field = db_field_map.get(field, f't.{field}')
                    
                    if operator == 'eq':
                        conditions.append(f"{db_field} = :{param_key}")
                        query_params[param_key] = value
                    elif operator == 'contains':
                        conditions.append(f"{db_field} LIKE :{param_key}")
                        query_params[param_key] = f"%{value}%"
                    elif operator == 'gt':
                        conditions.append(f"{db_field} > :{param_key}")
                        query_params[param_key] = value
                    elif operator == 'lt':
                        conditions.append(f"{db_field} < :{param_key}")
                        query_params[param_key] = value
            
            where_clause = " AND ".join(conditions)
            
            # 获取总数
            count_query = f"""
                SELECT COUNT(*) as total 
                FROM tasks t
                WHERE {where_clause}
            """
            count_result = await pg_conn.execute(text(count_query), query_params)
            total = count_result.fetchone().total
            
            # 构建排序
            sort_map = {
                'created_at': 't.created_at',
                'started_at': 't.started_at',
                'completed_at': 't.completed_at'
            }
            order_by = sort_map.get(sort_field, 't.created_at')
            order_direction = 'DESC' if sort_order == 'desc' else 'ASC'
            
            # 获取任务列表（从payload JSONB中提取数据）
            query = f"""
                SELECT 
                    t.stream_id as id,
                    t.payload::jsonb->'event_data'->>'__task_name' as task_name,
                    t.queue,
                    t.payload::jsonb->>'status' as status,
                    t.priority,
                    COALESCE((t.payload::jsonb->>'retry_count')::int, 0) as retry_count,
                    COALESCE((t.payload::jsonb->>'max_retry')::int, 3) as max_retry,
                    t.created_at,
                    (t.payload::jsonb->>'started_at')::timestamptz as started_at,
                    (t.payload::jsonb->>'completed_at')::timestamptz as completed_at,
                    t.payload::jsonb->>'worker_id' as worker_id,
                    t.payload::jsonb->>'error_message' as error_message,
                    (t.payload::jsonb->>'execution_time')::float as execution_time,
                    CASE 
                        WHEN t.payload::jsonb->>'completed_at' IS NOT NULL AND t.created_at IS NOT NULL 
                        THEN EXTRACT(EPOCH FROM ((t.payload::jsonb->>'completed_at')::timestamptz - t.created_at))
                        ELSE NULL 
                    END as duration,
                    t.scheduled_task_id,
                    t.source,
                    t.metadata
                FROM tasks t
                WHERE {where_clause}
                ORDER BY {order_by} {order_direction}
                LIMIT :limit OFFSET :offset
            """
            
            result = await pg_conn.execute(text(query), query_params)
            
            tasks = []
            for row in result:
                tasks.append({
                    "id": row.id,
                    "task_name": row.task_name or "unknown",
                    "queue": row.queue,
                    "status": row.status,
                    "priority": row.priority,
                    "retry_count": row.retry_count,
                    "max_retry": row.max_retry,
                    "created_at": row.created_at.isoformat() if row.created_at else None,
                    "started_at": row.started_at.isoformat() if row.started_at else None,
                    "completed_at": row.completed_at.isoformat() if row.completed_at else None,
                    "duration": round(row.duration, 2) if row.duration else None,
                    "execution_time": float(row.execution_time) if row.execution_time else None,
                    "worker_id": row.worker_id,
                    "error_message": row.error_message
                })
            
            return {
                "success": True,
                "data": tasks,
                "total": total
            }
    
    @staticmethod
    async def get_consumer_group_stats(namespace_data_access, namespace: str, group_name: str) -> Dict[str, Any]:
        """
        获取特定消费者组的详细统计
        
        Args:
            namespace_data_access: 命名空间数据访问实例
            namespace: 命名空间
            group_name: 消费者组名称
            
        Returns:
            消费者组统计数据
        """
        # 获取命名空间连接
        conn = await namespace_data_access.manager.get_connection(namespace)
        
        # 获取PostgreSQL会话
        if not conn.AsyncSessionLocal:
            raise ValueError("PostgreSQL not configured for this namespace")
        
        async with conn.AsyncSessionLocal() as session:
            # 查询消费者组的执行统计
            query = text("""
                WITH group_stats AS (
                    SELECT 
                        tr.consumer_group,
                        tr.task_name,
                        COUNT(*) as total_tasks,
                        COUNT(CASE WHEN tr.status = 'success' THEN 1 END) as success_count,
                        COUNT(CASE WHEN tr.status = 'failed' THEN 1 END) as failed_count,
                        COUNT(CASE WHEN tr.status = 'running' THEN 1 END) as running_count,
                        AVG(tr.execution_time) as avg_execution_time,
                        MIN(tr.execution_time) as min_execution_time,
                        MAX(tr.execution_time) as max_execution_time,
                        AVG(tr.duration) as avg_duration,
                        MIN(tr.started_at) as first_task_time,
                        MAX(tr.completed_at) as last_task_time
                    FROM task_runs tr
                    WHERE tr.consumer_group = :group_name
                        AND tr.started_at > NOW() - INTERVAL '24 hours'
                    GROUP BY tr.consumer_group, tr.task_name
                ),
                hourly_stats AS (
                    SELECT 
                        DATE_TRUNC('hour', tr.started_at) as hour,
                        COUNT(*) as task_count,
                        AVG(tr.execution_time) as avg_exec_time
                    FROM task_runs tr
                    WHERE tr.consumer_group = :group_name
                        AND tr.started_at > NOW() - INTERVAL '24 hours'
                    GROUP BY DATE_TRUNC('hour', tr.started_at)
                    ORDER BY hour
                )
                SELECT 
                    (SELECT row_to_json(gs) FROM group_stats gs) as summary,
                    (SELECT json_agg(hs) FROM hourly_stats hs) as hourly_trend
            """)
            
            result = await session.execute(query, {'group_name': group_name})
            row = result.fetchone()
            
            if not row or not row.summary:
                return {
                    "success": True,
                    "data": {
                        "group_name": group_name,
                        "summary": {},
                        "hourly_trend": []
                    }
                }
            
            return {
                "success": True,
                "data": {
                    "group_name": group_name,
                    "summary": row.summary,
                    "hourly_trend": row.hourly_trend or []
                }
            }
    
    @staticmethod
    async def get_stream_backlog(
        data_access,
        namespace: str,
        stream_name: Optional[str] = None,
        hours: int = 24
    ) -> Dict[str, Any]:
        """
        获取Stream积压监控数据
        
        Args:
            data_access: 数据访问层实例
            namespace: 命名空间
            stream_name: 可选，指定stream名称
            hours: 查询最近多少小时的数据
            
        Returns:
            Stream积压数据
        """
        # 计算时间范围
        end_time = datetime.now(timezone.utc)
        start_time = end_time - timedelta(hours=hours)
        
        async with data_access.AsyncSessionLocal() as session:
            # 构建查询
            if stream_name:
                query = text("""
                    SELECT 
                        stream_name,
                        consumer_group,
                        last_published_offset,
                        last_delivered_offset,
                        last_acked_offset,
                        pending_count,
                        backlog_undelivered,
                        backlog_unprocessed,
                        created_at
                    FROM stream_backlog_monitor
                    WHERE namespace = :namespace
                        AND stream_name = :stream_name
                        AND created_at >= :start_time
                        AND created_at <= :end_time
                    ORDER BY created_at DESC
                    LIMIT 1000
                """)
                params = {
                    'namespace': namespace,
                    'stream_name': stream_name,
                    'start_time': start_time,
                    'end_time': end_time
                }
            else:
                # 获取最新的所有stream数据
                query = text("""
                    SELECT DISTINCT ON (stream_name, consumer_group)
                        stream_name,
                        consumer_group,
                        last_published_offset,
                        last_delivered_offset,
                        last_acked_offset,
                        pending_count,
                        backlog_undelivered,
                        backlog_unprocessed,
                        created_at
                    FROM stream_backlog_monitor
                    WHERE namespace = :namespace
                        AND created_at >= :start_time
                    ORDER BY stream_name, consumer_group, created_at DESC
                """)
                params = {
                    'namespace': namespace,
                    'start_time': start_time
                }
            
            result = await session.execute(query, params)
            rows = result.fetchall()
            
            # 格式化数据
            data = []
            for row in rows:
                data.append({
                    'stream_name': row.stream_name,
                    'consumer_group': row.consumer_group,
                    'last_published_offset': row.last_published_offset,
                    'last_delivered_offset': row.last_delivered_offset,
                    'last_acked_offset': row.last_acked_offset,
                    'pending_count': row.pending_count,
                    'backlog_undelivered': row.backlog_undelivered,
                    'backlog_unprocessed': row.backlog_unprocessed,
                    'created_at': row.created_at.isoformat() if row.created_at else None
                })
            
            return {
                'success': True,
                'data': data,
                'total': len(data)
            }
    
    @staticmethod
    async def get_stream_backlog_summary(data_access, namespace: str) -> Dict[str, Any]:
        """
        获取Stream积压监控汇总数据
        
        Args:
            data_access: 数据访问层实例
            namespace: 命名空间
            
        Returns:
            汇总数据
        """
        async with data_access.AsyncSessionLocal() as session:
            # 获取最新的汇总数据
            query = text("""
                WITH latest_data AS (
                    SELECT DISTINCT ON (stream_name, consumer_group)
                        stream_name,
                        consumer_group,
                        backlog_undelivered,
                        backlog_unprocessed,
                        pending_count
                    FROM stream_backlog_monitor
                    WHERE namespace = :namespace
                        AND created_at >= NOW() - INTERVAL '1 hour'
                    ORDER BY stream_name, consumer_group, created_at DESC
                )
                SELECT 
                    COUNT(DISTINCT stream_name) as total_streams,
                    COUNT(DISTINCT consumer_group) as total_groups,
                    SUM(backlog_unprocessed) as total_backlog,
                    SUM(pending_count) as total_pending,
                    MAX(backlog_unprocessed) as max_backlog
                FROM latest_data
            """)
            
            result = await session.execute(query, {'namespace': namespace})
            row = result.fetchone()
            
            if row:
                return {
                    'success': True,
                    'data': {
                        'total_streams': row.total_streams or 0,
                        'total_groups': row.total_groups or 0,
                        'total_backlog': row.total_backlog or 0,
                        'total_pending': row.total_pending or 0,
                        'max_backlog': row.max_backlog or 0
                    }
                }
            else:
                return {
                    'success': True,
                    'data': {
                        'total_streams': 0,
                        'total_groups': 0,
                        'total_backlog': 0,
                        'total_pending': 0,
                        'max_backlog': 0
                    }
                }


#!/usr/bin/env python
"""
JetTask CLI - 命令行接口
"""
import asyncio
import click
import sys
import os
import importlib
import importlib.util
import multiprocessing
from pathlib import Path

# 处理直接运行时的路径问题
if __name__ == '__main__':
    parent_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    if parent_dir not in sys.path:
        sys.path.insert(0, parent_dir)

from jettask.config.nacos_config import config
from jettask.config.env_loader import EnvLoader
from jettask.core.app_importer import import_app, AppImporter


@click.group()
@click.version_option(version='0.1.0', prog_name='JetTask')
@click.option('--env-file', '-e', envvar='JETTASK_ENV_FILE',
              help='环境变量文件路径 (.env 格式)，可通过 JETTASK_ENV_FILE 环境变量指定')
@click.option('--use-nacos', is_flag=True,
              help='从Nacos配置中心读取配置')
@click.option('--nacos-server',
              help='Nacos服务器地址 (如: 127.0.0.1:8848)')
@click.option('--nacos-namespace',
              help='Nacos命名空间')
@click.option('--nacos-username',
              help='Nacos用户名')
@click.option('--nacos-password',
              help='Nacos密码')
@click.option('--nacos-group',
              help='Nacos配置组')
@click.option('--nacos-data-id',
              help='Nacos配置ID（api/webui-consumer/scheduler 需要）')
@click.pass_context
def cli(ctx, env_file, use_nacos, nacos_server, nacos_namespace,
        nacos_username, nacos_password, nacos_group, nacos_data_id):
    """JetTask - 高性能分布式任务队列系统

    配置加载优先级（从高到低）：
    1. 命令行参数
    2. 环境变量
    3. .env文件
    4. 默认值

    Nacos 必需参数：
      --nacos-server     Nacos服务器地址
      --nacos-namespace  Nacos命名空间

    Nacos 可选参数：
      --nacos-group      配置组（默认: DEFAULT_GROUP）
      --nacos-data-id    配置ID（api/webui-consumer/scheduler 需要）
      --nacos-username   用户名
      --nacos-password   密码

    说明：
      • worker 命令不使用全局 Nacos 配置，而是通过 app 读取配置
      • api/webui-consumer/scheduler 需要 nacos-data-id 来读取应用配置

    示例：
    \b
      # 使用.env文件
      jettask -e .env worker --queues default

      # 使用Nacos（从.env读取配置）
      jettask --use-nacos worker --queues default

      # 使用Nacos（命令行指定最小配置）
      jettask --use-nacos \\
        --nacos-server 127.0.0.1:8848 \\
        --nacos-namespace prod \\
        worker --queues default

      # 混合使用：先加载.env，再从Nacos覆盖
      jettask -e .env --use-nacos worker --queues default
    """
    # 在所有命令执行前加载环境变量
    ctx.ensure_object(dict)

    # 初始化 context（即使不使用 Nacos 也要设置）
    ctx.obj['use_nacos'] = use_nacos
    ctx.obj['nacos_data_id'] = None

    loader = EnvLoader()

    # 如果手动指定了env文件，只加载指定的文件；否则自动搜索加载
    if env_file:
        # 手动指定了文件，直接加载，不进行自动搜索
        try:
            loader.load_env_file(env_file, override=True)
            click.echo(f"✓ 已加载环境变量文件: {env_file}")
        except FileNotFoundError:
            click.echo(f"Error: 环境变量文件不存在: {env_file}", err=True)
            raise click.Abort()
        except Exception as e:
            click.echo(f"Error: 加载环境变量文件失败: {e}", err=True)
            raise click.Abort()
    else:
        # 未指定文件，自动搜索并加载 .env, .env.{ENV}, .env.local
        loader.auto_load()

    # 3. 如果启用了Nacos，从Nacos读取配置并更新环境变量
    if use_nacos:
        try:
            # 在加载 .env 之后，再从环境变量读取 Nacos 配置
            # 优先级: 命令行参数 > .env 文件中的环境变量
            nacos_server = nacos_server or os.environ.get('NACOS_SERVER')
            nacos_namespace = nacos_namespace or os.environ.get('NACOS_NAMESPACE')
            nacos_username = nacos_username or os.environ.get('NACOS_USERNAME')
            nacos_password = nacos_password or os.environ.get('NACOS_PASSWORD')
            nacos_group = nacos_group or os.environ.get('NACOS_GROUP', 'DEFAULT_GROUP')
            nacos_data_id = nacos_data_id or os.environ.get('NACOS_DATA_ID')

            # 更新 Nacos 配置到 context
            ctx.obj['nacos_data_id'] = nacos_data_id

            # 检查必需的配置是否完整（nacos_data_id 改为可选）
            if not all([nacos_server, nacos_namespace, nacos_group]):
                missing = []
                if not nacos_server: missing.append("--nacos-server 或 NACOS_SERVER")
                if not nacos_namespace: missing.append("--nacos-namespace 或 NACOS_NAMESPACE")
                if not nacos_group: missing.append("--nacos-group 或 NACOS_GROUP")

                click.echo("Error: Nacos配置不完整，缺少以下配置:", err=True)
                for item in missing:
                    click.echo(f"  - {item}", err=True)
                raise click.Abort()

            # 将命令行参数或环境变量的Nacos配置写入os.environ
            # 这样nacos_config模块才能读取到
            os.environ['NACOS_SERVER'] = nacos_server
            os.environ['NACOS_NAMESPACE'] = nacos_namespace
            os.environ['NACOS_GROUP'] = nacos_group
            if nacos_data_id:
                os.environ['NACOS_DATA_ID'] = nacos_data_id
            if nacos_username:
                os.environ['NACOS_USERNAME'] = nacos_username
            if nacos_password:
                os.environ['NACOS_PASSWORD'] = nacos_password

            click.echo(f"正在从Nacos加载配置 ({nacos_server}/{nacos_namespace})...")

            # 加载Nacos配置
            nacos_pg_url = config.config.get('JETTASK_PG_URL')
            if not nacos_pg_url:
                # 如果没有直接的URL，尝试从独立的配置项构建
                pg_host = config.config.get('PG_DB_HOST', 'localhost')
                pg_port = config.config.get('PG_DB_PORT', 5432)
                pg_user = config.config.get('PG_DB_USERNAME', 'jettask')
                pg_password = config.config.get('PG_DB_PASSWORD', '123456')
                pg_database = config.config.get('PG_DB_DATABASE', 'jettask')
                nacos_pg_url = f'postgresql://{pg_user}:{pg_password}@{pg_host}:{pg_port}/{pg_database}'

            # 将Nacos配置写入环境变量（会覆盖.env中的配置）
            if nacos_pg_url:
                os.environ['JETTASK_PG_URL'] = nacos_pg_url

            # 将其他Nacos配置也设置到环境变量中
            for key, value in config.config.items():
                if isinstance(value, (str, int, float, bool)):
                    os.environ[key] = str(value)

            click.echo(f"✓ 从Nacos加载配置成功")
        except click.Abort:
            raise
        except Exception as e:
            click.echo(f"Error: 从Nacos加载配置失败: {e}", err=True)
            import traceback
            traceback.print_exc()
            raise click.Abort()

    # 保存loader到context，供子命令使用
    ctx.obj['env_loader'] = loader
    ctx.obj['use_nacos'] = use_nacos

@cli.command()
@click.option('--host', default='0.0.0.0', envvar='JETTASK_API_HOST', help='服务器监听地址')
@click.option('--port', default=8001, type=int, envvar='JETTASK_API_PORT', help='服务器监听端口')
@click.option('--redis-url', envvar='JETTASK_REDIS_URL', help='Redis连接URL')
@click.option('--redis-prefix', envvar='JETTASK_REDIS_PREFIX', help='Redis键前缀')
@click.option('--jettask-pg-url', envvar='JETTASK_PG_URL', help='PostgreSQL连接URL')
@click.option('--reload', is_flag=True, envvar='JETTASK_API_RELOAD', help='启用自动重载')
@click.option('--log-level', default='info', envvar='JETTASK_LOG_LEVEL',
              type=click.Choice(['debug', 'info', 'warning', 'error']),
              help='日志级别')
@click.pass_context
def api(ctx, host, port, redis_url, redis_prefix, jettask_pg_url, reload, log_level):
    """启动 API 服务和监控界面

    示例:
    \b
      # 使用默认配置启动
      jettask api

      # 指定配置
      jettask api --host 0.0.0.0 --port 8080 \
        --redis-url redis://localhost:6379/0 \
        --redis-prefix my_app \
        --jettask-pg-url postgresql://user:pass@host/db

      # 使用Nacos配置中心（全局参数）
      jettask --use-nacos api

      # 通过环境变量配置（.env文件）
      jettask -e .env api

      # 启用开发模式（自动重载）
      jettask api --reload --log-level debug
    """
    import os
    import uvicorn

    # Click的envvar已经自动处理了环境变量读取
    # 全局的--use-nacos也已经处理了Nacos配置加载

    # 获取use_nacos状态（从全局context）
    use_nacos = ctx.obj.get('use_nacos', False)

    # 检查必需参数
    if not jettask_pg_url:
        raise ValueError(
            "必须提供 --jettask-pg-url 或在环境变量中设置 JETTASK_PG_URL\n"
            "或使用 jettask --use-nacos api 从Nacos加载配置"
        )

    if not redis_url:
        raise ValueError(
            "必须提供 --redis-url 或在环境变量中设置 JETTASK_REDIS_URL\n"
            "或使用 jettask --use-nacos api 从Nacos加载配置"
        )

    # 设置环境变量（供应用内部使用）
    os.environ['JETTASK_REDIS_URL'] = redis_url
    if redis_prefix:
        os.environ['JETTASK_REDIS_PREFIX'] = redis_prefix
    os.environ['JETTASK_PG_URL'] = jettask_pg_url
    os.environ['USE_NACOS'] = 'true' if use_nacos else 'false'
    
    # 使用标准应用模块
    app_module = "jettask.webui.app:app"
    click.echo(f"Starting JetTask API Server on {host}:{port}")
    
    # 显示配置信息
    click.echo("=" * 60)
    click.echo("API Server Configuration")
    click.echo("=" * 60)
    click.echo(f"Host:         {host}")
    click.echo(f"Port:         {port}")
    click.echo(f"Redis URL:    {redis_url}")
    click.echo(f"Redis Prefix: {redis_prefix or 'jettask (default)'}")
    click.echo(f"Database:     {jettask_pg_url}")
    click.echo(f"Auto-reload:  {reload}")
    click.echo(f"Log level:    {log_level}")
    click.echo(f"Nacos:        {'Enabled' if use_nacos else 'Disabled'}")
    click.echo("=" * 60)
    click.echo(f"API Endpoint: http://{host}:{port}/api")
    click.echo(f"WebUI:        http://{host}:{port}/")
    click.echo("=" * 60)
    
    # 启动服务器
    try:
        uvicorn.run(
            app_module,
            host=host,
            port=port,
            log_level=log_level,
            reload=reload
        )
    except KeyboardInterrupt:
        click.echo("\nShutting down API Server...")
    except Exception as e:
        click.echo(f"Error starting API Server: {e}", err=True)
        sys.exit(1)


def load_module_from_path(module_path: str):
    """从文件路径加载 Python 模块"""
    path = Path(module_path).resolve()
    
    if not path.exists():
        raise FileNotFoundError(f"Module file not found: {module_path}")
    
    # 获取模块名
    module_name = path.stem
    
    # 加载模块
    spec = importlib.util.spec_from_file_location(module_name, path)
    if spec is None:
        raise ImportError(f"Cannot load module from {module_path}")
    
    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)
    
    return module

def find_jettask_app(module):
    """在模块中查找 Jettask 实例"""
    from jettask import Jettask
    
    # 查找模块中的 Jettask 实例
    for name in dir(module):
        obj = getattr(module, name)
        if isinstance(obj, Jettask):
            return obj
    
    # 如果没有找到，尝试查找名为 'app' 的变量
    if hasattr(module, 'app'):
        obj = getattr(module, 'app')
        if isinstance(obj, Jettask):
            return obj
    
    return None

@cli.command()
@click.option('--app', '-a', 'app_str', envvar='JETTASK_APP',
              help='应用位置 (如: module:app, path/to/file.py:app, 或目录名)')
@click.option('--tasks', '-t', envvar='JETTASK_TASKS', help='任务名称（逗号分隔，如: task1,task2）')
@click.option('--concurrency', '-c', type=int, envvar='JETTASK_CONCURRENCY', help='并发数（默认为CPU核心数的一半）')
@click.option('--prefetch', '-p', type=int, default=100, envvar='JETTASK_PREFETCH', help='预取倍数')
def worker(app_str, tasks, concurrency, prefetch):
    """启动任务处理 Worker

    示例:
    \b
      # 使用 -a 或 --app 参数指定 app，使用任务名称
      jettask worker -a main:app --tasks my_task
      jettask worker --app tasks.py:app --tasks task1,task2
      jettask worker -a myapp --tasks high_priority_task,normal_task

      # 自动发现 app（从当前目录的 app.py 或 main.py）
      jettask worker --tasks my_task

      # 使用环境变量
      export JETTASK_APP=myapp:app
      jettask worker --tasks my_task

      # 加载环境变量文件（全局 -e 参数）
      jettask -e .env worker --tasks my_task
      jettask --env-file production.env worker -a main:app --tasks task1,task2

      # 或使用环境变量
      export JETTASK_ENV_FILE=.env
      jettask worker --tasks my_task
    """

    # Click的envvar已经自动处理了环境变量读取

    # 检查必需的 tasks 参数
    if not tasks:
        click.echo("Error: 必须指定 --tasks 参数或设置 JETTASK_TASKS 环境变量", err=True)
        click.echo("\n示例:", err=True)
        click.echo("  jettask worker --tasks my_task", err=True)
        click.echo("  jettask worker -t task1,task2", err=True)
        click.echo("  export JETTASK_TASKS=my_task && jettask worker", err=True)
        raise click.Abort()

    # 设置默认 concurrency 值（如果没有通过CLI或环境变量指定）
    cpu_count = multiprocessing.cpu_count()
    if concurrency is None:
        concurrency = max(1, cpu_count // 4)
        click.echo(f"Using default concurrency: {concurrency} (1/4 of {cpu_count} CPU cores)")
    if concurrency>cpu_count:
        click.echo(f"Error: Specified concurrency {concurrency} exceeds CPU cores {cpu_count}", err=True)
        raise click.Abort()

    # 加载应用
    try:
        if app_str:
            click.echo(f"Loading app from: {app_str}")
            app = import_app(app_str)
        else:
            click.echo("Auto-discovering Jettask app...")
            click.echo("Searching in: app.py, main.py, server.py, worker.py")
            app = import_app()  # 自动发现
        
        
        # app.redis_prefix = app.redis_prefix or 'jettask'
        # 显示应用信息
        app_info = AppImporter.get_app_info(app)
        click.echo(f"\nFound Jettask app:")
        click.echo(f"  Tasks: {app_info['tasks']} registered")
        
        if app_info.get('task_names') and app_info['tasks'] > 0:
            task_preview = app_info['task_names'][:3]
            click.echo(f"  Names: {', '.join(task_preview)}" + 
                      (f" (+{app_info['tasks'] - 3} more)" if app_info['tasks'] > 3 else ""))
    except ImportError as e:
        import traceback
        click.echo(f"Error: Failed to import app: {e}", err=True)
        
        # 始终显示完整的堆栈跟踪，帮助用户定位问题
        click.echo("\n" + "=" * 60, err=True)
        click.echo("Full traceback:", err=True)
        click.echo("=" * 60, err=True)
        traceback.print_exc()
        click.echo("=" * 60, err=True)
        
        click.echo("\nTips:", err=True)
        click.echo("  - Check if there are syntax errors in your code", err=True)
        click.echo("  - Verify all imports in your module are available", err=True)
        click.echo("  - Specify app location: jettask worker myapp:app", err=True)
        click.echo("  - Or set environment variable: export JETTASK_APP=myapp:app", err=True)
        click.echo("  - Or ensure app.py or main.py exists in current directory", err=True)
        sys.exit(1)
    except Exception as e:
        import traceback
        traceback.print_exc()
        click.echo(f"Error loading app: {e}", err=True)
        
        # 对于所有异常都显示堆栈信息
        click.echo("\n" + "=" * 60, err=True)
        click.echo("Full traceback:", err=True)
        click.echo("=" * 60, err=True)
        traceback.print_exc()
        click.echo("=" * 60, err=True)
        
        click.echo("\nThis might be a bug in JetTask or your application.", err=True)
        click.echo("Please check the traceback above for details.", err=True)
        sys.exit(1)
    
    # 解析任务列表（支持逗号分隔）
    task_list = [t.strip() for t in tasks.split(',') if t.strip()]

    # 从 app 实例中获取实际配置
    redis_url = app.redis_url if hasattr(app, 'redis_url') else 'Not configured'
    redis_prefix = app.redis_prefix if hasattr(app, 'redis_prefix') else 'jettask'
    consumer_strategy = app.consumer_strategy if hasattr(app, 'consumer_strategy') else 'heartbeat'

    # 显示配置信息
    click.echo("=" * 60)
    click.echo("JetTask Worker Configuration")
    click.echo("=" * 60)
    click.echo(f"App:          {app_str}")
    click.echo(f"Redis URL:    {redis_url}")
    click.echo(f"Redis Prefix: {redis_prefix}")
    click.echo(f"Strategy:     {consumer_strategy}")
    click.echo(f"Tasks:        {', '.join(task_list)}")
    click.echo(f"Concurrency:  {concurrency}")
    click.echo(f"Prefetch:     {prefetch}")
    click.echo("=" * 60)

    # 启动 Worker
    try:
        click.echo(f"Starting worker...")
        app.start(
            tasks=task_list,
            concurrency=concurrency,
            prefetch_multiplier=prefetch
        )
    except KeyboardInterrupt:
        click.echo("\nShutting down worker...")
    except Exception as e:
        click.echo(f"Error starting worker: {e}", err=True)
        sys.exit(1)

@cli.command('webui-consumer')
@click.option('--task-center', '-tc', envvar='JETTASK_CENTER_URL', required=True,
              help='任务中心URL，如: http://localhost:8001 或 http://localhost:8001/api/namespaces/default')
@click.option('--check-interval', type=int, default=30,
              help='命名空间检测间隔（秒），仅多命名空间模式使用，默认30秒')
@click.option('--concurrency', '-c', type=int, default=4,
              help='并发数（每个命名空间的 worker 进程数），默认4')
@click.option('--prefetch', '-p', type=int, default=1000,
              help='预取倍数（控制每次从队列预取的消息数），默认1000')
@click.option('--debug', is_flag=True, help='启用调试模式')
def webui_consumer(task_center, check_interval, concurrency, prefetch, debug):
    """启动 PostgreSQL 数据消费者（自动识别单/多命名空间）

    从 Redis 队列消费任务并持久化到 PostgreSQL 数据库。
    根据URL格式自动判断运行模式:
    - 单命名空间: http://localhost:8001/api/namespaces/{name}
    - 多命名空间: http://localhost:8001 或 http://localhost:8001/api

    示例:
    \b
      # 为所有命名空间启动消费者（自动检测）
      jettask webui-consumer --task-center http://localhost:8001
      jettask webui-consumer --task-center http://localhost:8001/api

      # 为单个命名空间启动消费者
      jettask webui-consumer --task-center http://localhost:8001/api/namespaces/default

      # 自定义配置
      jettask webui-consumer --task-center http://localhost:8001 --check-interval 60

      # 使用环境变量
      export JETTASK_CENTER_URL=http://localhost:8001
      jettask webui-consumer
    """
    import asyncio
    from jettask.persistence.manager import UnifiedConsumerManager

    # 运行消费者管理器
    async def run_manager():
        """运行统一的消费者管理器"""
        manager = UnifiedConsumerManager(
            task_center_url=task_center,
            check_interval=check_interval,
            concurrency=concurrency,
            prefetch_multiplier=prefetch,
            debug=debug
        )
        await manager.run()

    try:
        asyncio.run(run_manager())
    except KeyboardInterrupt:
        click.echo("\n✓ 消费者已关闭")

# @cli.command()
# def monitor():
#     """启动系统监控器"""
#     click.echo("Starting JetTask Monitor")
#     # 已删除 run_monitor 模块
#     click.echo("Monitor command is deprecated and has been removed.")

@cli.command()
@click.option('--pg-url', envvar='JETTASK_PG_URL',
              help='PostgreSQL连接URL (如: postgresql://user:pass@localhost/db)')
def init(pg_url):
    """初始化数据库和配置

    示例:
    \b
      # 使用命令行参数
      jettask init --pg-url postgresql://user:pass@localhost/jettask

      # 使用环境变量
      export JETTASK_PG_URL=postgresql://user:pass@localhost/jettask
      jettask init

      # 使用 .env 文件
      jettask -e .env init
    """
    click.echo("Initializing JetTask...")

    import os

    # 如果提供了 PG URL，设置到环境变量
    if pg_url:
        os.environ['JETTASK_PG_URL'] = pg_url
        click.echo(f"Using PostgreSQL: {pg_url.split('@')[1] if '@' in pg_url else pg_url}")

    # 初始化数据库
    from jettask.utils.db_init import init_database
    click.echo("\nInitializing database...")
    init_database()

    click.echo("\n✓ JetTask initialized successfully!")

@cli.command()
def status():
    """显示系统状态"""
    click.echo("JetTask System Status")
    click.echo("=" * 50)
    
    # 检查 Redis 连接
    try:
        from jettask.db.connector import get_sync_redis_client
        r = get_sync_redis_client('redis://localhost:6379/0', decode_responses=True)
        r.ping()
        click.echo("✓ Redis: Connected")
    except Exception as e:
        click.echo(f"✗ Redis: Not connected ({e})")
    
    # 检查 PostgreSQL 连接
    try:
        import psycopg2
        conn = psycopg2.connect(
            dbname=os.getenv('JETTASK_PG_DB', 'jettask'),
            user=os.getenv('JETTASK_PG_USER', 'jettask'),
            password=os.getenv('JETTASK_PG_PASSWORD', '123456'),
            host=os.getenv('JETTASK_PG_HOST', 'localhost'),
            port=os.getenv('JETTASK_PG_PORT', '5432')
        )
        conn.close()
        click.echo("✓ PostgreSQL: Connected")
    except:
        click.echo("✗ PostgreSQL: Not connected")
    
    click.echo("=" * 50)

@cli.command()
@click.option('--task-center', '-tc', envvar='JETTASK_CENTER_URL', required=True,
              help='任务中心URL，如: http://localhost:8001 或 http://localhost:8001/api/namespaces/default')
@click.option('--interval', '-i', type=float, default=0.1,
              help='调度器扫描间隔（秒），默认0.1秒')
@click.option('--batch-size', '-b', type=int, default=100,
              help='每批处理的最大任务数，默认100')
@click.option('--check-interval', type=int, default=30,
              help='命名空间检测间隔（秒），仅多命名空间模式使用，默认30秒')
@click.option('--debug', is_flag=True, help='启用调试模式')
def scheduler(task_center, interval, batch_size, check_interval, debug):
    """启动定时任务调度器（自动识别单/多命名空间）

    根据URL格式自动判断运行模式:
    - 单命名空间: http://localhost:8001/api/namespaces/{name}
    - 多命名空间: http://localhost:8001 或 http://localhost:8001/api

    示例:
    \b
      # 为所有命名空间启动调度器（自动检测）
      jettask scheduler --task-center http://localhost:8001
      jettask scheduler --task-center http://localhost:8001/api

      # 为单个命名空间启动调度器
      jettask scheduler --task-center http://localhost:8001/api/namespaces/default

      # 自定义配置
      jettask scheduler --task-center http://localhost:8001 --check-interval 60 --interval 0.5

      # 使用环境变量
      export JETTASK_CENTER_URL=http://localhost:8001
      jettask scheduler
    """
    import asyncio
    from jettask.scheduler.manager import UnifiedSchedulerManager

    # 运行调度器管理器
    async def run_manager():
        """运行统一的调度器管理器"""
        manager = UnifiedSchedulerManager(
            task_center_url=task_center,
            scan_interval=interval,
            batch_size=batch_size,
            check_interval=check_interval,
            debug=debug
        )
        await manager.run()

    try:
        asyncio.run(run_manager())
    except KeyboardInterrupt:
        click.echo("\nShutdown complete")

@cli.command()
@click.option('--port', default=8080, help='前端服务器端口')
@click.option('--host', default='0.0.0.0', help='前端服务器监听地址')
@click.option('--api-url', default='http://localhost:8001', help='后端 API 服务器地址')
@click.option('--auto-install', is_flag=True, default=True, help='自动安装缺失的依赖')
@click.option('--force-install', is_flag=True, help='强制重新安装依赖')
@click.option('--build-only', is_flag=True, help='仅构建生产版本，不启动服务器')
def frontend(port, host, api_url, auto_install, force_install, build_only):
    """启动 WebUI 前端界面
    
    启动生产版本服务器：
    1. 检查 Node.js 和 npm 是否安装
    2. 在包目录安装依赖并构建
    3. 启动生产版本服务器
    
    示例：
      # 启动生产版本服务器（默认）
      jettask frontend
      
      # 指定端口
      jettask frontend --port 3000
      
      # 指定后端API地址
      jettask frontend --api-url http://192.168.1.100:8001
      
      # 仅构建生产版本
      jettask frontend --build-only
      
      # 强制重新安装依赖
      jettask frontend --force-install
    """
    import subprocess
    import shutil
    from pathlib import Path
    
    # 获取前端目录路径
    frontend_dir = Path(__file__).parent.parent / "webui" / "frontend"
    if not frontend_dir.exists():
        click.echo(f"错误：前端目录不存在: {frontend_dir}", err=True)
        sys.exit(1)
    
    # 检查 Node.js 是否安装
    node_cmd = shutil.which('node')
    if not node_cmd:
        click.echo("错误：未检测到 Node.js 环境", err=True)
        click.echo("\n请先安装 Node.js:")
        click.echo("  - Ubuntu/Debian: sudo apt-get install nodejs npm")
        click.echo("  - macOS: brew install node")
        click.echo("  - Windows: 从 https://nodejs.org 下载安装")
        click.echo("  - 或使用 nvm: curl -o- https://raw.githubusercontent.com/nvm-sh/nvm/v0.39.0/install.sh | bash")
        sys.exit(1)
    
    # 检查 npm 是否安装
    npm_cmd = shutil.which('npm')
    if not npm_cmd:
        click.echo("错误：未检测到 npm", err=True)
        click.echo("\n请安装 npm:")
        click.echo("  - Ubuntu/Debian: sudo apt-get install npm")
        click.echo("  - 或重新安装 Node.js (包含 npm)")
        sys.exit(1)
    
    # 显示版本信息
    try:
        node_version = subprocess.check_output([node_cmd, '--version'], text=True).strip()
        npm_version = subprocess.check_output([npm_cmd, '--version'], text=True).strip()
        click.echo(f"检测到 Node.js {node_version}, npm {npm_version}")
    except subprocess.CalledProcessError:
        pass
    
    # 切换到前端目录
    os.chdir(frontend_dir)
    click.echo(f"前端目录: {frontend_dir}")
    
    # 检查 package.json 是否存在
    package_json = frontend_dir / "package.json"
    if not package_json.exists():
        click.echo("错误：package.json 文件不存在", err=True)
        sys.exit(1)
    
    # 检查 node_modules 是否存在
    node_modules = frontend_dir / "node_modules"
    need_install = not node_modules.exists() or force_install
    
    if need_install and auto_install:
        click.echo("\n正在安装依赖...")
        try:
            # 清理旧的 node_modules（如果强制安装）
            if force_install and node_modules.exists():
                click.echo("清理旧的 node_modules...")
                shutil.rmtree(node_modules)
            
            # 清理 package-lock.json（如果有问题）
            lock_file = frontend_dir / "package-lock.json"
            if force_install and lock_file.exists():
                lock_file.unlink()
            
            # 运行 npm install
            subprocess.run([npm_cmd, 'install'], check=True)
            click.echo("✓ 依赖安装完成")
        except subprocess.CalledProcessError as e:
            click.echo(f"错误：依赖安装失败 - {e}", err=True)
            click.echo("\n可以尝试:")
            click.echo("  1. 手动删除 node_modules 目录后重试")
            click.echo("  2. 使用 --force-install 参数强制重新安装")
            click.echo("  3. 检查网络连接和 npm 源设置")
            sys.exit(1)
    elif need_install and not auto_install:
        click.echo("警告：未检测到 node_modules，请运行 'npm install' 安装依赖")
        if not click.confirm("是否现在安装依赖？"):
            sys.exit(1)
    
    # 构建或启动
    try:
        # 构建生产版本
        # 注意：根据 vite.config.js，构建输出到 ../static/dist
        dist_dir = frontend_dir.parent / "static" / "dist"
        
        # 检查是否需要构建
        need_build = not dist_dir.exists() or build_only
        
        # 如果 dist 不存在或明确要求构建，则进行构建
        if need_build:
            click.echo("\n正在构建生产版本...")
            subprocess.run([npm_cmd, 'run', 'build'], check=True)
            
            if dist_dir.exists():
                # 统计文件
                files = list(dist_dir.rglob('*'))
                file_count = len([f for f in files if f.is_file()])
                total_size = sum(f.stat().st_size for f in files if f.is_file())
                click.echo(f"\n✓ 构建完成！")
                click.echo(f"  - 输出目录: {dist_dir}")
                click.echo(f"  - 文件数量: {file_count}")
                click.echo(f"  - 总大小: {total_size / 1024 / 1024:.2f} MB")
            else:
                click.echo("错误：构建失败，未生成输出目录", err=True)
                sys.exit(1)
        
        # 如果不是仅构建模式，启动生产服务器
        if not build_only:
            if not dist_dir.exists():
                click.echo("错误：未找到构建输出，请先运行构建", err=True)
                sys.exit(1)
            
            click.echo(f"\n正在启动生产版本服务器...")
            click.echo(f"  - 访问地址: http://localhost:{port}")
            if host != 'localhost':
                click.echo(f"  - 网络地址: http://{host}:{port}")
            click.echo(f"  - 静态文件: {dist_dir}")
            click.echo(f"  - API 地址: {api_url}")
            click.echo("\n按 Ctrl+C 停止服务器\n")
            
            # 使用 Python 内置的 HTTP 服务器提供静态文件
            import http.server
            import socketserver
            import urllib.request
            
            class ProxyHandler(http.server.SimpleHTTPRequestHandler):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, directory=str(dist_dir), **kwargs)
                
                def proxy_request(self, method='GET', body=None):
                    """代理 API 请求到后端服务器"""
                    if self.path.startswith('/api/') or self.path.startswith('/ws/'):
                        # 构建目标 URL
                        target_url = api_url + self.path
                        
                        try:
                            # 准备请求
                            req = urllib.request.Request(target_url, method=method)
                            
                            # 复制请求头
                            for header, value in self.headers.items():
                                if header.lower() not in ['host', 'connection']:
                                    req.add_header(header, value)
                            
                            # 添加请求体
                            if body:
                                req.data = body
                            
                            # 发送请求
                            with urllib.request.urlopen(req, timeout=30) as response:
                                # 返回响应
                                self.send_response(response.getcode())
                                for header, value in response.headers.items():
                                    if header.lower() not in ['connection', 'transfer-encoding']:
                                        self.send_header(header, value)
                                self.end_headers()
                                self.wfile.write(response.read())
                            return True
                            
                        except urllib.error.HTTPError as e:
                            self.send_response(e.code)
                            self.end_headers()
                            self.wfile.write(e.read())
                            return True
                        except Exception as e:
                            self.send_error(502, f"Bad Gateway: {str(e)}")
                            return True
                    return False
                
                def do_GET(self):
                    # 先尝试代理 API 请求
                    if self.proxy_request('GET'):
                        return
                    
                    # 对于 index.html，注入 API URL
                    if self.path == '/' or self.path == '/index.html' or (
                        self.path != '/' and not Path(dist_dir / self.path[1:]).exists()
                    ):
                        # 读取 index.html
                        index_path = dist_dir / 'index.html'
                        if index_path.exists():
                            with open(index_path, 'r', encoding='utf-8') as f:
                                html_content = f.read()
                            
                            # 注入 API URL 配置（使用相对路径，因为我们会代理）
                            api_config = f'''
                            <script>
                                window.JETTASK_API_URL = '';  // 使用相对路径，由服务器代理
                                console.log('API requests will be proxied by server');
                            </script>
                            '''
                            
                            # 在 </head> 标签前注入配置
                            html_content = html_content.replace('</head>', api_config + '</head>')
                            
                            # 发送响应
                            self.send_response(200)
                            self.send_header('Content-type', 'text/html')
                            self.send_header('Content-Length', str(len(html_content.encode('utf-8'))))
                            self.end_headers()
                            self.wfile.write(html_content.encode('utf-8'))
                            return
                    
                    # 其他文件正常处理
                    return super().do_GET()
                
                def do_POST(self):
                    # 读取请求体
                    content_length = int(self.headers.get('Content-Length', 0))
                    body = self.rfile.read(content_length) if content_length > 0 else None
                    
                    # 代理 POST 请求
                    if self.proxy_request('POST', body):
                        return
                    
                    # 不支持的请求
                    self.send_error(405, "Method Not Allowed")
                
                def do_PUT(self):
                    content_length = int(self.headers.get('Content-Length', 0))
                    body = self.rfile.read(content_length) if content_length > 0 else None
                    if self.proxy_request('PUT', body):
                        return
                    self.send_error(405, "Method Not Allowed")
                
                def do_DELETE(self):
                    if self.proxy_request('DELETE'):
                        return
                    self.send_error(405, "Method Not Allowed")
                
                def log_message(self, format, *args):
                    # 自定义日志格式
                    click.echo(f"[{self.log_date_time_string()}] {format % args}")
            
            # 创建服务器
            class ReuseAddrTCPServer(socketserver.TCPServer):
                allow_reuse_address = True
                allow_reuse_port = True
            
            try:
                click.echo(f"正在绑定到 {host}:{port}...")
                httpd = ReuseAddrTCPServer((host, port), ProxyHandler)
                click.echo(f"✓ 服务器已启动，监听 {host}:{port}")
                click.echo(f"✓ API 请求将被代理到 {api_url}")
                httpd.serve_forever()
            except OSError as e:
                if e.errno == 98:  # Address already in use
                    click.echo(f"错误：端口 {port} 已被占用，请尝试其他端口", err=True)
                else:
                    click.echo(f"错误：{e}", err=True)
                sys.exit(1)
            except KeyboardInterrupt:
                pass
    except subprocess.CalledProcessError as e:
        click.echo(f"错误：命令执行失败 - {e}", err=True)
        sys.exit(1)
    except KeyboardInterrupt:
        click.echo("\n停止前端服务器")

def main():
    """主入口函数"""
    try:
        cli()
    except KeyboardInterrupt:
        click.echo("\nOperation cancelled by user")
        sys.exit(0)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)

if __name__ == '__main__':
    main()


"""
API v1 路由模块集合
"""
from fastapi import APIRouter

# 导入8个主要模块的路由
from .overview import router as overview_router      # 概览
from .namespaces import router as namespaces_router  # 命名空间
from .queues import router as queues_router         # 队列
from .scheduled import router as scheduled_router    # 定时任务
from .alerts import router as alerts_router         # 告警
from .settings import router as settings_router      # 设置
from .workers import router as workers_router        # Worker 监控

# 创建 v1 总路由，添加统一的 /api/v1 前缀
api_router = APIRouter(prefix="/api/v1")

# 按功能模块组织路由
api_router.include_router(overview_router)    # 系统概览、健康检查
api_router.include_router(namespaces_router)  # 命名空间管理（独立路由）
api_router.include_router(queues_router)      # 队列管理、任务处理
api_router.include_router(scheduled_router)   # 定时任务管理
api_router.include_router(alerts_router)      # 告警规则管理
api_router.include_router(settings_router)    # 系统配置
api_router.include_router(workers_router)     # Worker 监控

__all__ = ['api_router']





"""
告警服务层
处理告警相关的业务逻辑
"""
from typing import Optional, List, Dict, Any
from datetime import datetime, timedelta, timezone
import logging

from jettask.schemas import AlertRuleRequest

logger = logging.getLogger(__name__)


class AlertService:
    """告警服务类"""
    
    @staticmethod
    def get_alert_rules(
        page: int = 1,
        page_size: int = 20,
        is_active: Optional[bool] = None
    ) -> Dict[str, Any]:
        """
        获取告警规则列表
        
        Args:
            page: 页码
            page_size: 每页大小
            is_active: 是否激活
            
        Returns:
            告警规则列表
        """
        # 模拟数据
        rules = AlertService._get_mock_rules()
        
        # 过滤
        if is_active is not None:
            rules = [r for r in rules if r["is_active"] == is_active]
        
        # 分页
        total = len(rules)
        start = (page - 1) * page_size
        end = start + page_size
        paginated_rules = rules[start:end]
        
        return {
            "success": True,
            "data": paginated_rules,
            "total": total,
            "page": page,
            "page_size": page_size
        }
    
    @staticmethod
    def create_alert_rule(request: AlertRuleRequest) -> Dict[str, Any]:
        """
        创建告警规则
        
        Args:
            request: 告警规则请求
            
        Returns:
            创建的告警规则
        """
        rule = {
            "id": f"rule_{datetime.now().strftime('%Y%m%d%H%M%S')}",
            "name": request.name,
            "rule_type": request.rule_type,
            "target_queues": request.target_queues,
            "condition": request.condition,
            "action_type": request.action_type,
            "action_config": request.action_config,
            "is_active": request.is_active,
            "description": request.description,
            "check_interval": request.check_interval,
            "created_at": datetime.now(timezone.utc).isoformat(),
            "last_triggered": None,
            "updated_at": None
        }
        
        # TODO: 实际应该保存到数据库
        
        return {
            "success": True,
            "data": rule,
            "message": "告警规则创建成功"
        }
    
    @staticmethod
    def update_alert_rule(rule_id: str, request: AlertRuleRequest) -> Dict[str, Any]:
        """
        更新告警规则
        
        Args:
            rule_id: 规则ID
            request: 告警规则请求
            
        Returns:
            更新后的告警规则
        """
        rule = {
            "id": rule_id,
            "name": request.name,
            "rule_type": request.rule_type,
            "target_queues": request.target_queues,
            "condition": request.condition,
            "action_type": request.action_type,
            "action_config": request.action_config,
            "is_active": request.is_active,
            "description": request.description,
            "check_interval": request.check_interval,
            "updated_at": datetime.now(timezone.utc).isoformat()
        }
        
        # TODO: 实际应该更新数据库
        
        return {
            "success": True,
            "data": rule,
            "message": "告警规则更新成功"
        }
    
    @staticmethod
    def delete_alert_rule(rule_id: str) -> Dict[str, Any]:
        """
        删除告警规则
        
        Args:
            rule_id: 规则ID
            
        Returns:
            删除结果
        """
        # TODO: 实际应该从数据库删除
        
        return {
            "success": True,
            "message": f"告警规则 {rule_id} 已删除"
        }
    
    @staticmethod
    def toggle_alert_rule(rule_id: str) -> Dict[str, Any]:
        """
        启用/禁用告警规则
        
        Args:
            rule_id: 规则ID
            
        Returns:
            切换结果
        """
        # TODO: 实际应该更新数据库中的状态
        # 这里模拟切换逻辑
        current_state = True  # 假设当前是激活状态
        new_state = not current_state
        
        return {
            "success": True,
            "data": {
                "id": rule_id,
                "is_active": new_state
            },
            "message": f"告警规则已{'启用' if new_state else '禁用'}"
        }
    
    @staticmethod
    def get_alert_history(
        rule_id: str,
        page: int = 1,
        page_size: int = 20
    ) -> Dict[str, Any]:
        """
        获取告警触发历史
        
        Args:
            rule_id: 规则ID
            page: 页码
            page_size: 每页大小
            
        Returns:
            告警历史记录
        """
        # 模拟告警历史数据
        history = AlertService._generate_mock_history(rule_id)
        
        # 分页
        total = len(history)
        start = (page - 1) * page_size
        end = start + page_size
        paginated_history = history[start:end]
        
        return {
            "success": True,
            "data": paginated_history,
            "total": total,
            "page": page,
            "page_size": page_size
        }
    
    @staticmethod
    def test_alert_rule(rule_id: str) -> Dict[str, Any]:
        """
        测试告警规则（发送测试通知）
        
        Args:
            rule_id: 规则ID
            
        Returns:
            测试结果
        """
        # TODO: 实际应该触发真实的通知发送
        
        return {
            "success": True,
            "message": "测试通知已发送",
            "data": {
                "rule_id": rule_id,
                "test_time": datetime.now(timezone.utc).isoformat(),
                "notification_result": {
                    "status": "success",
                    "response": {"code": 200, "message": "OK"}
                }
            }
        }
    
    @staticmethod
    def get_alert_statistics(namespace: Optional[str] = None) -> Dict[str, Any]:
        """
        获取告警统计信息
        
        Args:
            namespace: 命名空间（可选）
            
        Returns:
            告警统计数据
        """
        # 模拟统计数据
        return {
            "success": True,
            "data": {
                "total_rules": 15,
                "active_rules": 10,
                "inactive_rules": 5,
                "alerts_today": 25,
                "alerts_this_week": 156,
                "alerts_this_month": 892,
                "most_triggered_rule": {
                    "id": "rule_001",
                    "name": "队列积压告警",
                    "trigger_count": 42
                },
                "alert_trend": AlertService._generate_alert_trend(),
                "alert_by_type": {
                    "queue_size": 45,
                    "error_rate": 30,
                    "response_time": 15,
                    "custom": 10
                }
            }
        }
    
    @staticmethod
    def get_active_alerts(namespace: Optional[str] = None) -> List[Dict[str, Any]]:
        """
        获取当前活跃的告警
        
        Args:
            namespace: 命名空间（可选）
            
        Returns:
            活跃告警列表
        """
        # 模拟活跃告警
        return [
            {
                "id": "alert_active_001",
                "rule_id": "rule_001",
                "rule_name": "队列积压告警",
                "severity": "high",
                "triggered_at": (datetime.now(timezone.utc) - timedelta(hours=2)).isoformat(),
                "trigger_value": 1500,
                "threshold": 1000,
                "queue": "order_queue",
                "status": "active",
                "acknowledged": False
            },
            {
                "id": "alert_active_002",
                "rule_id": "rule_002",
                "rule_name": "高错误率告警",
                "severity": "critical",
                "triggered_at": (datetime.now(timezone.utc) - timedelta(minutes=30)).isoformat(),
                "trigger_value": 0.15,
                "threshold": 0.1,
                "queue": "payment_queue",
                "status": "active",
                "acknowledged": True,
                "acknowledged_by": "admin",
                "acknowledged_at": (datetime.now(timezone.utc) - timedelta(minutes=20)).isoformat()
            }
        ]
    
    @staticmethod
    def acknowledge_alert(alert_id: str, user: str = "system") -> Dict[str, Any]:
        """
        确认告警
        
        Args:
            alert_id: 告警ID
            user: 确认用户
            
        Returns:
            确认结果
        """
        # TODO: 实际应该更新数据库
        
        return {
            "success": True,
            "data": {
                "alert_id": alert_id,
                "acknowledged": True,
                "acknowledged_by": user,
                "acknowledged_at": datetime.now(timezone.utc).isoformat()
            },
            "message": "告警已确认"
        }
    
    @staticmethod
    def resolve_alert(alert_id: str, resolution_note: Optional[str] = None) -> Dict[str, Any]:
        """
        解决告警
        
        Args:
            alert_id: 告警ID
            resolution_note: 解决说明
            
        Returns:
            解决结果
        """
        # TODO: 实际应该更新数据库
        
        return {
            "success": True,
            "data": {
                "alert_id": alert_id,
                "status": "resolved",
                "resolved_at": datetime.now(timezone.utc).isoformat(),
                "resolution_note": resolution_note
            },
            "message": "告警已解决"
        }
    
    # ============ 私有辅助方法 ============
    
    @staticmethod
    def _get_mock_rules() -> List[Dict[str, Any]]:
        """获取模拟的告警规则数据"""
        return [
            {
                "id": "rule_001",
                "name": "队列积压告警",
                "rule_type": "queue_size",
                "target_queues": ["order_queue", "payment_queue"],
                "condition": {"threshold": 1000, "operator": ">"},
                "action_type": "webhook",
                "action_config": {"url": "https://example.com/webhook"},
                "is_active": True,
                "last_triggered": "2025-08-31T14:30:00Z",
                "created_at": "2025-08-01T10:00:00Z",
                "description": "当队列积压超过1000时触发告警"
            },
            {
                "id": "rule_002",
                "name": "高错误率告警",
                "rule_type": "error_rate",
                "target_queues": ["*"],
                "condition": {"threshold": 0.1, "operator": ">", "window": 300},
                "action_type": "email",
                "action_config": {"recipients": ["admin@example.com"]},
                "is_active": True,
                "last_triggered": None,
                "created_at": "2025-08-15T14:00:00Z",
                "description": "5分钟内错误率超过10%时告警"
            },
            {
                "id": "rule_003",
                "name": "响应时间告警",
                "rule_type": "response_time",
                "target_queues": ["api_queue"],
                "condition": {"threshold": 5000, "operator": ">", "percentile": 95},
                "action_type": "webhook",
                "action_config": {"url": "https://slack.com/webhook"},
                "is_active": False,
                "last_triggered": "2025-08-30T09:00:00Z",
                "created_at": "2025-07-01T08:00:00Z",
                "description": "95分位响应时间超过5秒时告警"
            },
            {
                "id": "rule_004",
                "name": "内存使用告警",
                "rule_type": "memory_usage",
                "target_queues": ["*"],
                "condition": {"threshold": 80, "operator": ">"},
                "action_type": "sms",
                "action_config": {"phone": "+1234567890"},
                "is_active": True,
                "last_triggered": "2025-08-29T16:00:00Z",
                "created_at": "2025-06-15T12:00:00Z",
                "description": "内存使用率超过80%时告警"
            }
        ]
    
    @staticmethod
    def _generate_mock_history(rule_id: str) -> List[Dict[str, Any]]:
        """生成模拟的告警历史"""
        history = []
        for i in range(1, 21):
            triggered_at = datetime.now(timezone.utc) - timedelta(hours=i*3)
            is_resolved = i % 3 != 0  # 每3个告警有1个未解决
            
            alert = {
                "id": f"alert_{rule_id}_{i}",
                "rule_id": rule_id,
                "triggered_at": triggered_at.isoformat(),
                "trigger_value": 1200 + i * 50,
                "threshold": 1000,
                "status": "resolved" if is_resolved else "active",
                "notification_sent": True,
                "notification_response": {"status": "success"}
            }
            
            if is_resolved:
                alert["resolved_at"] = (triggered_at + timedelta(hours=1)).isoformat()
                alert["resolution_note"] = f"自动恢复 - 值降至阈值以下"
            
            history.append(alert)
        
        return history
    
    @staticmethod
    def _generate_alert_trend() -> List[Dict[str, Any]]:
        """生成告警趋势数据"""
        trend = []
        now = datetime.now(timezone.utc)
        
        for i in range(7):
            date = (now - timedelta(days=6-i)).date()
            trend.append({
                "date": date.isoformat(),
                "count": 10 + i * 3 + (i % 2) * 5  # 模拟波动
            })
        
        return trend


"""PostgreSQL Consumer - 基于通配符队列的新实现

完全替换旧的 consumer.py 实现，使用 Jettask 通配符队列功能。
"""

import time
import logging
from datetime import datetime, timezone

from jettask import Jettask
from jettask.core.context import TaskContext
from jettask.db.connector import get_pg_engine_and_factory, DBConfig
from .buffer import BatchBuffer
from .persistence import TaskPersistence

logger = logging.getLogger(__name__)


def _decode_redis_field(value, field_type='str'):
    """解析 Redis 字段值（处理 bytes/str 类型）

    Args:
        value: Redis 返回的值（可能是 bytes 或 str）
        field_type: 目标类型 ('str', 'int', 'float')

    Returns:
        解析后的值，如果解析失败返回 None
    """
    if not value:
        return None

    # 如果是 bytes，先解码为 str
    if isinstance(value, bytes):
        try:
            value = value.decode('utf-8')
        except Exception:
            return None

    # 类型转换
    try:
        if field_type == 'int':
            return int(value) if value else 0
        elif field_type == 'float':
            return float(value) if value else None
        else:  # str
            return value
    except (ValueError, TypeError):
        return None


def _extract_task_name_from_consumer(consumer: str) -> str:
    """从 consumer 字段提取 task_name

    consumer 格式: YYDG-15b50489-9274-robust_bench2:8
    提取逻辑: 用'-'分割取最后一个，再用':'分割取第一个
    结果: robust_bench2

    Args:
        consumer: consumer 字段值

    Returns:
        task_name，如果解析失败返回 None
    """
    if not consumer:
        return None

    try:
        # 用 '-' 分割，取最后一个
        last_part = consumer.split('-')[-1]
        # 用 ':' 分割，取第一个
        task_name = last_part.split(':')[0]
        return task_name if task_name else None
    except (IndexError, AttributeError):
        logger.warning(f"Failed to extract task_name from consumer: {consumer}")
        return None


def _parse_task_info(task_info: dict) -> dict:
    """批量解析任务信息字段

    Args:
        task_info: Redis hgetall 返回的任务信息字典

    Returns:
        解析后的字段字典
    """
    consumer = _decode_redis_field(task_info.get(b'consumer'), 'str')

    # 🔧 从 consumer 字段提取 task_name
    task_name = _extract_task_name_from_consumer(consumer)

    return {
        'retries': _decode_redis_field(task_info.get(b'retries'), 'int'),
        'started_at': _decode_redis_field(task_info.get(b'started_at'), 'float'),
        'completed_at': _decode_redis_field(task_info.get(b'completed_at'), 'float'),
        'consumer': consumer,
        'status': _decode_redis_field(task_info.get(b'status'), 'str'),
        'result': task_info.get(b'result'),  # 保持原始 bytes
        'error': task_info.get(b'exception') or task_info.get(b'error'),  # 保持原始 bytes
        'task_name': task_name,  # 🔧 添加提取的 task_name
    }


def _extract_event_id_from_task_id(task_id: str) -> str:
    """从 task_id 中提取 event_id

    task_id 格式: prefix:TASK:event_id:queue:task_name

    Args:
        task_id: 任务 ID

    Returns:
        event_id (stream_id)，如果格式无效返回 None
    """
    if not task_id:
        return None

    parts = task_id.split(':')
    if len(parts) >= 3:
        return parts[2]  # 提取 event_id

    return None


def _extract_task_name_from_task_id(task_id: str) -> str:
    """从 task_id 中提取 task_name

    task_id 格式: prefix:TASK:event_id:queue:task_name

    Args:
        task_id: 任务 ID

    Returns:
        task_name，如果格式无效返回 None
    """
    if not task_id:
        return None

    parts = task_id.split(':')
    if len(parts) >= 5:
        return parts[4]  # 提取 task_name

    return None


class PostgreSQLConsumer:
    """PostgreSQL Consumer - 基于通配符队列

    核心特性：
    1. 使用 @app.task(queue='*') 监听所有队列
    2. 使用 @app.task(queue='TASK_CHANGES') 处理状态更新
    3. 批量 INSERT 和 UPDATE
    4. 自动队列发现（Jettask 内置）
    """

    def __init__(
        self,
        pg_config,  # 可以是字典或配置对象
        redis_config,  # 可以是字典或配置对象
        prefix: str = "jettask",
        namespace_id: str = None,
        namespace_name: str = None,
        batch_size: int = 1000,
        flush_interval: float = 5.0
    ):
        """初始化 PG Consumer

        Args:
            pg_config: PostgreSQL配置（字典或对象）
            redis_config: Redis配置（字典或对象）
            prefix: Redis键前缀
            node_id: 节点ID（兼容旧接口，不使用）
            namespace_id: 命名空间ID
            namespace_name: 命名空间名称
            enable_backlog_monitor: 是否启用积压监控（兼容旧接口，不使用）
            backlog_monitor_interval: 积压监控间隔（兼容旧接口，不使用）
            batch_size: 批量大小
            flush_interval: 刷新间隔（秒）
        """
        self.pg_config = pg_config
        self.redis_config = redis_config
        self.redis_prefix = prefix
        self.namespace_id = namespace_id
        self.namespace_name = namespace_name or "default"

        # 构建 Redis URL（兼容字典和对象两种格式）
        if isinstance(redis_config, dict):
            # 字典格式 - 优先使用 'url' 字段
            redis_url = redis_config.get('url') or redis_config.get('redis_url')
            if not redis_url:
                # 从独立字段构建
                password = redis_config.get('password', '')
                host = redis_config.get('host', 'localhost')
                port = redis_config.get('port', 6379)
                db = redis_config.get('db', 0)
                redis_url = f"redis://"
                if password:
                    redis_url += f":{password}@"
                redis_url += f"{host}:{port}/{db}"
        else:
            # 对象格式
            redis_url = f"redis://"
            if hasattr(redis_config, 'password') and redis_config.password:
                redis_url += f":{redis_config.password}@"
            redis_url += f"{redis_config.host}:{redis_config.port}/{redis_config.db}"

        self.redis_url = redis_url
        logger.debug(f"构建 Redis URL: {redis_url}")

        # 数据库引擎和会话（将在 start 时初始化）
        self.async_engine = None
        self.AsyncSessionLocal = None
        self.db_manager = None

        # 创建 Jettask 应用
        self.app = Jettask(
            redis_url=redis_url,
            redis_prefix=prefix
        )

        # 创建两个独立的批量缓冲区
        # 1. INSERT 缓冲区（用于新任务持久化）
        self.insert_buffer = BatchBuffer(
            max_size=batch_size,
            max_delay=flush_interval,
            operation_type='insert'
        )

        # 2. UPDATE 缓冲区（用于任务状态更新）
        self.update_buffer = BatchBuffer(
            max_size=batch_size // 2,  # 状态更新通常更频繁，用较小的批次
            max_delay=flush_interval,
            operation_type='update',
            redis_client_getter=lambda: self.app.async_binary_redis  # 批量获取任务信息
        )

        # 注册任务
        self._register_tasks()

        # 运行控制
        self._running = False

        # auto flush 启动标志（在 worker 进程中懒加载启动）
        self._auto_flush_started = False

    async def _ensure_auto_flush_started(self):
        """确保 auto flush 在 worker 进程中启动（只启动一次）"""
        if not self._auto_flush_started:
            logger.info("[Worker进程] 启动缓冲区自动刷新任务...")
            await self.insert_buffer.start_auto_flush(self.db_manager)
            await self.update_buffer.start_auto_flush(self.db_manager)
            self._auto_flush_started = True
            logger.info("[Worker进程] ✓ 缓冲区自动刷新任务已启动")

    def _register_tasks(self):
        """注册任务处理器"""
        # 创建闭包函数来访问实例属性
        consumer = self  # 捕获 self 引用

        @self.app.task(queue='*', auto_ack=False)
        async def _handle_persist_task(ctx: TaskContext, **kwargs):
            print(f'收到_handle_persist_task任务 {kwargs=} {ctx.metadata=}')
            return await consumer._do_handle_persist_task(ctx, **kwargs)

        @self.app.task(queue='TASK_CHANGES', auto_ack=False)
        async def _handle_status_update(ctx: TaskContext, **kwargs):
            print(f'收到_handle_status_update任务 {kwargs=} {ctx.metadata=}')

            return await consumer._do_handle_status_update(ctx, **kwargs)

    async def _do_handle_persist_task(self, ctx: TaskContext, **kwargs):
        """处理任务持久化（INSERT）

        使用通配符 queue='*' 监听所有队列
        Jettask 会自动发现新队列并开始消费

        Args:
            ctx: Jettask 自动注入的任务上下文（包含 queue, event_id 等）
            **kwargs: 任务的原始数据字段
        """
        # 🔧 确保 auto flush 在 worker 进程中启动（懒加载）
        await self._ensure_auto_flush_started()

        # 添加关键日志，确认方法被调用
        logger.info(f"[持久化任务] 收到消息 - 队列: {ctx.queue}, Stream ID: {ctx.event_id}, task_name: {kwargs.get('task_name')}, metadata: {ctx.metadata}")

        # 跳过 TASK_CHANGES 队列（由另一个任务处理）
        if ctx.queue == f'{self.redis_prefix}:QUEUE:TASK_CHANGES':
            logger.debug(f"[持久化任务] 跳过 TASK_CHANGES 队列: {ctx.event_id}")
            ctx.acks([ctx.event_id])
            return

        try:

            # 🔧 从 ctx.metadata 中提取元数据
            metadata = ctx.metadata or {}

            trigger_time = metadata.get('trigger_time', time.time())
            if isinstance(trigger_time, (str, bytes)):
                trigger_time = float(trigger_time)

            priority = metadata.get('priority', 0)
            if priority and isinstance(priority, (str, bytes)):
                priority = int(priority)
            elif priority is None:
                priority = 0

            # 提取 delay 参数
            delay = metadata.get('delay', 0)
            if delay and isinstance(delay, (str, bytes)):
                delay = float(delay)
            elif delay is None:
                delay = 0

            scheduled_task_id = metadata.get('scheduled_task_id')

            payload = {
                'args': kwargs.get('args', []),
                'kwargs': kwargs.get('kwargs', []),
            }

            record = {
                'stream_id': ctx.event_id,
                'queue': ctx.queue.replace(f'{self.redis_prefix}:QUEUE:', ''),
                'payload': payload,
                'priority': priority,
                'delay': delay,
                'created_at': datetime.fromtimestamp(trigger_time, tz=timezone.utc),
                'scheduled_task_id': scheduled_task_id,
                'namespace': self.namespace_name,
                'source': 'scheduler' if scheduled_task_id else 'redis_stream',
            }

            # 添加到缓冲区（不立即处理，不立即 ACK）
            await self.insert_buffer.add(record, ctx)
            logger.debug(f"[持久化任务] 已添加到缓冲区，当前大小: {len(self.insert_buffer.records)}/{self.insert_buffer.max_size}")

            # 检查是否需要刷新（批量大小或超时）
            if self.insert_buffer.should_flush():
                logger.info(f"[持久化任务] 触发刷新，缓冲区大小: {len(self.insert_buffer.records)}")
                await self.insert_buffer.flush(self.db_manager)

            # 同时检查 UPDATE 缓冲区是否需要刷新（利用这次机会）
            if self.update_buffer.should_flush():
                await self.update_buffer.flush(self.db_manager)

        except Exception as e:
            logger.error(f"持久化任务失败: {e}", exc_info=True)
            # 出错也要 ACK，避免消息堆积
            ctx.acks([ctx.event_id])

    async def _do_handle_status_update(self, ctx: TaskContext, **kwargs):
        """处理任务状态更新（UPDATE）

        消费 TASK_CHANGES 队列，批量更新数据库中的任务状态

        Args:
            ctx: Jettask 自动注入的任务上下文
            **kwargs: 任务的原始数据字段（包含 task_id）
        """
        # 添加关键日志，确认方法被调用
        logger.info(f"[状态更新] 收到消息 - 队列: {ctx.queue}, Stream ID: {ctx.event_id}, kwargs: {kwargs}")

        # 🔧 确保 auto flush 在 worker 进程中启动（懒加载）
        await self._ensure_auto_flush_started()

        try:
            # 从消息中获取 task_id
            task_id = kwargs.get('task_id')
            if not task_id:
                logger.warning(f"TASK_CHANGES 消息缺少 task_id: {ctx.event_id}")
                ctx.acks([ctx.event_id])
                return

            # 从 task_id 中提取 event_id (stream_id) 和 task_name
            event_id = _extract_event_id_from_task_id(task_id)
            if not event_id:
                logger.error(f"无效的 task_id 格式: {task_id}")
                ctx.acks([ctx.event_id])
                return

            task_name = _extract_task_name_from_task_id(task_id)

            # 只保存 task_id 和 task_name，延迟到批量刷新时再获取任务信息
            update_record = {
                'task_id': task_id,
                'stream_id': event_id,
                'task_name': task_name,
            }

            # 添加到状态更新缓冲区
            await self.update_buffer.add(update_record, ctx)
            logger.debug(f"[状态更新] 已添加到缓冲区，当前大小: {len(self.update_buffer.records)}/{self.update_buffer.max_size}")

            # 检查是否需要刷新（批量大小或超时）
            if self.update_buffer.should_flush():
                logger.info(f"[状态更新] 触发刷新，缓冲区大小: {len(self.update_buffer.records)}")
                await self.update_buffer.flush(self.db_manager)

            # 同时检查 INSERT 缓冲区是否需要刷新（利用这次机会）
            if self.insert_buffer.should_flush():
                await self.insert_buffer.flush(self.db_manager)

        except Exception as e:
            logger.error(f"更新任务状态失败: {e}", exc_info=True)
            # 出错也要 ACK
            ctx.acks([ctx.event_id])

    async def start(self, concurrency: int = 4, prefetch_multiplier: int = 1):
        """启动 Consumer

        Args:
            concurrency: 并发数
        """
        logger.info(f"Starting PostgreSQL consumer (wildcard queue mode)")
        logger.info(f"Namespace: {self.namespace_name} ({self.namespace_id or 'N/A'})")

        # 1. 使用 connector.py 统一管理数据库连接
        # 解析 PostgreSQL 配置为标准 DSN
        dsn = DBConfig.parse_pg_config(self.pg_config)

        # 使用全局单例引擎和会话工厂
        self.async_engine, self.AsyncSessionLocal = get_pg_engine_and_factory(
            dsn,
            pool_size=50,
            max_overflow=20,
            pool_pre_ping=True,
            pool_recycle=300,
            echo=False
        )

        logger.debug(f"使用全局 PostgreSQL 连接池: {dsn[:50]}...")

        # 2. 初始化任务持久化管理器
        self.db_manager = TaskPersistence(
            async_session_local=self.AsyncSessionLocal,
            namespace_id=self.namespace_id,
            namespace_name=self.namespace_name
        )

        # 3. 设置运行状态
        self._running = True

        # 4. 注意：不在主进程中启动 auto flush
        # auto flush 会在 worker 子进程中懒加载启动（首次任务执行时）
        # 这样避免进程隔离问题（主进程的 asyncio.Task 无法在子进程中运行）

        # 5. 在启动 worker 前执行一些同步的准备工作（从 app.start() 中提取）
        # 标记 worker 已启动
        self.app._worker_started = True

        # 如果配置了任务中心且配置尚未加载，从任务中心获取配置
        if self.app.task_center and self.app.task_center.is_enabled and not self.app._task_center_config:
            self.app._load_config_from_task_center()

        # 注册所有待注册的限流配置到 Redis
        logger.info("正在注册待注册的限流配置...")
        self.app._apply_pending_rate_limits()

        # 注册清理处理器（只在启动worker时注册）
        self.app._setup_cleanup_handlers()

        if self.app.consumer_strategy == "pod":
            raise ValueError("multi_asyncio模式下无法使用pod策略")

        # 启动 Worker（使用通配符队列）
        logger.info("=" * 60)
        logger.info(f"启动 PG Consumer (通配符队列模式)")
        logger.info("=" * 60)
        logger.info(f"命名空间: {self.namespace_name} ({self.namespace_id or 'N/A'})")
        logger.info(f"监听队列: * (所有队列) + TASK_CHANGES (状态更新)")
        logger.info(f"INSERT 批量: {self.insert_buffer.max_size} 条")
        logger.info(f"UPDATE 批量: {self.update_buffer.max_size} 条")
        logger.info(f"刷新间隔: {self.insert_buffer.max_delay} 秒")
        logger.info(f"并发数: {concurrency}")
        logger.info("=" * 60)

        try:
            # 启动 Worker
            # 注意：这里调用 _start() 而不是 start()，因为：
            # - app.start() 是同步方法，内部使用 asyncio.run()
            # - app._start() 是异步方法，可以在已有的事件循环中使用 await
            # - 我们的 consumer.start() 是异步的，所以必须调用 _start()

            # 获取已注册任务的名称
            task_names = list(self.app._tasks.keys())
            logger.info(f"已注册的任务: {task_names}")

            await self.app._start(
                tasks=task_names,  # 🎯 关键：传递任务名称列表
                concurrency=concurrency,
                prefetch_multiplier=prefetch_multiplier
            )
        finally:
            await self.stop()

    async def stop(self):
        """停止 Consumer"""
        logger.info("停止 PG Consumer...")
        self._running = False

        # 停止自动刷新任务（会自动执行最后一次刷新）
        try:
            await self.insert_buffer.stop_auto_flush()
            await self.update_buffer.stop_auto_flush()
            logger.info("✓ 缓冲区自动刷新任务已停止")
        except Exception as e:
            logger.error(f"停止自动刷新任务失败: {e}")

        # 注意：不关闭数据库引擎，因为它是全局单例，由 connector.py 管理
        # 多个 consumer 实例可能共享同一个引擎

        # 打印统计信息
        insert_stats = self.insert_buffer.get_stats()
        update_stats = self.update_buffer.get_stats()

        logger.info("=" * 60)
        logger.info("PG Consumer 统计信息")
        logger.info("=" * 60)
        logger.info(f"INSERT: 总计 {insert_stats['total_flushed']} 条, "
                   f"刷新 {insert_stats['flush_count']} 次, "
                   f"平均 {insert_stats['avg_per_flush']} 条/次")
        logger.info(f"UPDATE: 总计 {update_stats['total_flushed']} 条, "
                   f"刷新 {update_stats['flush_count']} 次, "
                   f"平均 {update_stats['avg_per_flush']} 条/次")
        logger.info("=" * 60)

        logger.info("PG Consumer 已停止")



"""
数据库基础配置

使用 SQLAlchemy 2.0 的异步API
"""
from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession, async_sessionmaker
from sqlalchemy.orm import declarative_base
from sqlalchemy.pool import NullPool
import logging

logger = logging.getLogger(__name__)

# 创建基类
Base = declarative_base()

# 全局引擎和会话工厂
_engine = None
_async_session_factory = None


def get_engine(database_url: str, **kwargs):
    """
    获取或创建数据库引擎

    Args:
        database_url: 数据库连接URL（如 postgresql+asyncpg://user:pass@host/db）
        **kwargs: 其他引擎参数

    Returns:
        AsyncEngine: 异步数据库引擎
    """
    global _engine

    if _engine is None:
        # 确保使用正确的异步驱动
        if database_url and 'postgresql://' in database_url:
            database_url = database_url.replace('postgresql://', 'postgresql+asyncpg://')

        # 默认配置
        engine_kwargs = {
            'echo': kwargs.pop('echo', False),
            'pool_pre_ping': kwargs.pop('pool_pre_ping', True),
            'poolclass': kwargs.pop('poolclass', NullPool),  # 使用NullPool避免连接池问题
        }
        engine_kwargs.update(kwargs)

        _engine = create_async_engine(database_url, **engine_kwargs)
        logger.info(f"数据库引擎已创建: {database_url.split('@')[-1]}")

    return _engine


def get_session_factory(database_url: str = None, **kwargs):
    """
    获取或创建会话工厂

    Args:
        database_url: 数据库连接URL
        **kwargs: 其他引擎参数

    Returns:
        async_sessionmaker: 异步会话工厂
    """
    global _async_session_factory

    if _async_session_factory is None:
        if database_url is None:
            raise ValueError("数据库URL未提供")

        engine = get_engine(database_url, **kwargs)
        _async_session_factory = async_sessionmaker(
            engine,
            class_=AsyncSession,
            expire_on_commit=False,
        )

    return _async_session_factory


def get_session(database_url: str = None, **kwargs):
    """
    获取数据库会话（上下文管理器）

    Args:
        database_url: 数据库连接URL
        **kwargs: 其他引擎参数

    Returns:
        AsyncSession: 异步数据库会话上下文管理器

    Example:
        async with get_session(db_url) as session:
            result = await session.execute(select(Task))
            tasks = result.scalars().all()
    """
    factory = get_session_factory(database_url, **kwargs)
    return factory()


async def init_db(database_url: str, **kwargs):
    """
    初始化数据库（创建所有表）

    Args:
        database_url: 数据库连接URL
        **kwargs: 其他引擎参数

    Example:
        await init_db('postgresql+asyncpg://user:pass@localhost/jettask')
    """
    # 导入所有模型以注册到 Base.metadata
    from .models import Task, ScheduledTask, TaskExecutionHistory  # noqa: F401

    engine = get_engine(database_url, **kwargs)

    async with engine.begin() as conn:
        # 创建所有表
        await conn.run_sync(Base.metadata.create_all)
        logger.info("数据库表已创建/更新")


async def drop_all(database_url: str, **kwargs):
    """
    删除所有表（谨慎使用！）

    Args:
        database_url: 数据库连接URL
        **kwargs: 其他引擎参数
    """
    # 导入所有模型以注册到 Base.metadata
    from .models import Task, ScheduledTask, TaskExecutionHistory  # noqa: F401

    engine = get_engine(database_url, **kwargs)

    async with engine.begin() as conn:
        await conn.run_sync(Base.metadata.drop_all)
        logger.warning("所有数据库表已删除")



"""
任务相关的数据模型
"""
from typing import Optional, List, Dict, Any
from datetime import datetime
from pydantic import BaseModel, Field


class TasksRequest(BaseModel):
    """任务查询请求模型"""
    queue_name: str = Field(..., description="队列名称")
    page: int = Field(default=1, ge=1, description="页码")
    page_size: int = Field(default=20, ge=1, le=100, description="每页大小")
    filters: Optional[List[Dict]] = Field(default=[], description="筛选条件")
    time_range: Optional[str] = Field(None, description="时间范围，如 '15m', '1h', '7d'")
    start_time: Optional[str] = Field(None, description="开始时间")
    end_time: Optional[str] = Field(None, description="结束时间")


class TaskInfo(BaseModel):
    """任务信息模型"""
    id: str = Field(..., description="任务ID")
    queue_name: str = Field(..., description="队列名称")
    status: str = Field(..., description="任务状态")
    created_at: datetime = Field(..., description="创建时间")
    started_at: Optional[datetime] = Field(None, description="开始时间")
    completed_at: Optional[datetime] = Field(None, description="完成时间")
    worker_id: Optional[str] = Field(None, description="工作节点ID")
    task_name: Optional[str] = Field(None, description="任务名称")
    task_data: Optional[Dict[str, Any]] = Field(None, description="任务数据")
    result: Optional[Any] = Field(None, description="任务结果")
    error: Optional[str] = Field(None, description="错误信息")
    retry_count: int = Field(default=0, description="重试次数")
    execution_time: Optional[float] = Field(None, description="执行时间（秒）")
    priority: Optional[int] = Field(None, description="优先级")
    consumer_group: Optional[str] = Field(None, description="消费者组")
    
    class Config:
        json_encoders = {
            datetime: lambda v: v.isoformat() if v else None
        }


class TaskDetailResponse(BaseModel):
    """任务详细信息响应模型"""
    success: bool = Field(default=True, description="请求是否成功")
    data: Optional[TaskInfo] = Field(None, description="任务详细信息")
    error: Optional[str] = Field(None, description="错误信息")
    
    class Config:
        json_encoders = {
            datetime: lambda v: v.isoformat() if v else None
        }


class TaskActionRequest(BaseModel):
    """任务操作请求模型"""
    task_ids: List[str] = Field(..., min_items=1, description="任务ID列表")
    action: str = Field(..., description="操作类型", pattern="^(retry|cancel|delete)$")
    reason: Optional[str] = Field(None, description="操作原因")


class TaskListResponse(BaseModel):
    """任务列表响应模型"""
    success: bool = Field(default=True, description="请求是否成功")
    data: List[TaskInfo] = Field(default=[], description="任务列表")
    total: int = Field(default=0, description="总记录数")
    page: int = Field(default=1, description="当前页码")
    page_size: int = Field(default=20, description="每页大小")
    error: Optional[str] = Field(None, description="错误信息")


"""
重构示例 - 展示如何使用 namespace 依赖注入

这个文件展示了如何将现有的API路由重构为使用新的依赖注入方案
"""
from fastapi import APIRouter, Depends, Query, HTTPException
from typing import Optional
import logging

from jettask.utils.namespace_dep import NamespaceContext, get_namespace_context

logger = logging.getLogger(__name__)

# 创建路由
router = APIRouter(prefix="/example", tags=["example"])


# ============================================================================
# 示例1: 基础用法 - 获取队列列表
# ============================================================================

@router.get(
    "/{namespace}/queues",
    summary="获取队列列表（重构示例）",
    description="展示如何使用NamespaceContext依赖注入"
)
async def get_queues_refactored(
    ns: NamespaceContext = Depends(get_namespace_context)
):
    """
    获取命名空间下的所有队列

    这个示例展示了最基础的用法：
    1. 使用 Depends(get_namespace_context) 注入命名空间上下文
    2. 通过 ns 对象访问 Redis 客户端
    3. 执行业务逻辑

    对比旧代码:
    - 不需要手动从 request.app.state 获取 namespace_data_access
    - 不需要手动调用 get_connection
    - 不需要手动处理错误（404/500等）
    - 代码从 20+ 行减少到 10 行
    """
    # 获取 Redis 客户端（自动处理连接）
    redis_client = await ns.get_redis_client()

    try:
        # 获取所有队列键
        queue_keys = await redis_client.keys(f"{ns.redis_prefix}:QUEUE:*")

        # 提取队列名称
        queues = []
        for key in queue_keys:
            queue_name = key.replace(f"{ns.redis_prefix}:QUEUE:", "")
            # 获取队列长度
            length = await redis_client.xlen(key)
            queues.append({
                "name": queue_name,
                "length": length
            })

        return {
            "success": True,
            "namespace": ns.namespace_name,
            "queues": queues
        }
    finally:
        await redis_client.aclose()


# ============================================================================
# 示例2: 带查询参数 - 获取队列统计
# ============================================================================

@router.get(
    "/{namespace}/queue-stats",
    summary="获取队列统计（重构示例）",
    description="展示如何组合使用路径参数、查询参数和依赖注入"
)
async def get_queue_stats_refactored(
    queue: Optional[str] = Query(None, description="队列名称，为空则返回所有队列统计"),
    time_range: str = Query("1h", description="时间范围", regex="^(1h|6h|24h|7d)$"),
    ns: NamespaceContext = Depends(get_namespace_context)
):
    """
    获取队列统计信息

    这个示例展示了如何组合使用：
    1. 路径参数 (namespace) - 自动注入到 ns
    2. 查询参数 (queue, time_range) - 正常声明
    3. 依赖注入 (ns) - 提供数据库访问

    关键点:
    - 路径参数 namespace 不需要在函数参数中声明
    - 查询参数正常使用 Query() 声明
    - 依赖注入参数放在最后
    """
    redis_client = await ns.get_redis_client()

    try:
        stats = {
            "namespace": ns.namespace_name,
            "time_range": time_range,
            "timestamp": __import__('datetime').datetime.now().isoformat()
        }

        if queue:
            # 获取指定队列的统计
            queue_key = f"{ns.redis_prefix}:QUEUE:{queue}"
            exists = await redis_client.exists(queue_key)

            if not exists:
                raise HTTPException(status_code=404, detail=f"队列 '{queue}' 不存在")

            stats["queue"] = queue
            stats["length"] = await redis_client.xlen(queue_key)

            # 获取消费者组信息
            try:
                groups = await redis_client.xinfo_groups(queue_key)
                stats["consumer_groups"] = len(groups)
                stats["total_pending"] = sum(g.get('pending', 0) for g in groups)
            except:
                stats["consumer_groups"] = 0
                stats["total_pending"] = 0
        else:
            # 获取所有队列的汇总统计
            queue_keys = await redis_client.keys(f"{ns.redis_prefix}:QUEUE:*")
            total_length = 0
            total_groups = 0

            for key in queue_keys:
                total_length += await redis_client.xlen(key)
                try:
                    groups = await redis_client.xinfo_groups(key)
                    total_groups += len(groups)
                except:
                    pass

            stats["total_queues"] = len(queue_keys)
            stats["total_length"] = total_length
            stats["total_consumer_groups"] = total_groups

        return {"success": True, "stats": stats}

    finally:
        await redis_client.aclose()


# ============================================================================
# 示例3: 使用 PostgreSQL - 获取定时任务
# ============================================================================

@router.get(
    "/{namespace}/scheduled-tasks",
    summary="获取定时任务列表（重构示例）",
    description="展示如何同时使用 Redis 和 PostgreSQL"
)
async def get_scheduled_tasks_refactored(
    page: int = Query(1, ge=1, description="页码"),
    page_size: int = Query(20, ge=1, le=100, description="每页数量"),
    is_active: Optional[bool] = Query(None, description="是否只返回激活的任务"),
    ns: NamespaceContext = Depends(get_namespace_context)
):
    """
    获取定时任务列表

    这个示例展示了如何使用 PostgreSQL：
    1. 使用 ns.get_pg_session() 获取数据库会话
    2. 执行 SQL 查询
    3. 格式化返回结果

    注意事项:
    - PostgreSQL 会话使用 async with 上下文管理
    - 会自动处理事务提交和回滚
    - 查询参数自动绑定，防止 SQL 注入
    """
    # 检查是否配置了 PostgreSQL
    if not ns.pg_config:
        raise HTTPException(
            status_code=503,
            detail=f"命名空间 '{ns.namespace_name}' 未配置 PostgreSQL"
        )

    async with await ns.get_pg_session() as session:
        from sqlalchemy import text

        # 构建查询条件
        conditions = ["namespace = :namespace"]
        params = {
            "namespace": ns.namespace_name,
            "limit": page_size,
            "offset": (page - 1) * page_size
        }

        if is_active is not None:
            conditions.append("enabled = :is_active")
            params["is_active"] = is_active

        where_clause = " AND ".join(conditions)

        # 查询定时任务
        query = text(f"""
            SELECT
                id, task_name as name, queue_name,
                enabled, next_run_time,
                cron_expression, interval_seconds,
                created_at, updated_at
            FROM scheduled_tasks
            WHERE {where_clause}
            ORDER BY next_run_time ASC NULLS LAST
            LIMIT :limit OFFSET :offset
        """)

        result = await session.execute(query, params)
        tasks = result.fetchall()

        # 获取总数
        count_query = text(f"""
            SELECT COUNT(*) FROM scheduled_tasks WHERE {where_clause}
        """)
        count_result = await session.execute(
            count_query,
            {k: v for k, v in params.items() if k not in ['limit', 'offset']}
        )
        total = count_result.scalar()

        # 格式化结果
        task_list = []
        for task in tasks:
            task_list.append({
                "id": task.id,
                "name": task.name,
                "queue_name": task.queue_name,
                "enabled": task.enabled,
                "next_run_time": task.next_run_time.isoformat() if task.next_run_time else None,
                "schedule": task.cron_expression or f"{task.interval_seconds}s",
                "created_at": task.created_at.isoformat() if task.created_at else None
            })

        return {
            "success": True,
            "namespace": ns.namespace_name,
            "tasks": task_list,
            "total": total,
            "page": page,
            "page_size": page_size,
            "has_more": (page * page_size) < total
        }


# ============================================================================
# 示例4: 复杂业务逻辑 - 队列健康检查
# ============================================================================

@router.post(
    "/{namespace}/queue/{queue_name}/health-check",
    summary="队列健康检查（重构示例）",
    description="展示如何在一个端点中同时使用 Redis 和 PostgreSQL 进行复杂的业务逻辑处理"
)
async def queue_health_check_refactored(
    queue_name: str,
    ns: NamespaceContext = Depends(get_namespace_context)
):
    """
    队列健康检查

    这个示例展示了复杂的业务逻辑处理：
    1. 从 Redis 获取队列实时状态
    2. 从 PostgreSQL 获取历史统计
    3. 综合分析并返回健康度评估

    重构后的优势:
    - 不需要担心连接管理
    - 专注于业务逻辑
    - 代码更简洁易读
    """
    health_report = {
        "namespace": ns.namespace_name,
        "queue": queue_name,
        "timestamp": __import__('datetime').datetime.now().isoformat(),
        "status": "unknown",
        "issues": [],
        "recommendations": []
    }

    redis_client = await ns.get_redis_client()

    try:
        # 1. 检查队列是否存在
        queue_key = f"{ns.redis_prefix}:QUEUE:{queue_name}"
        exists = await redis_client.exists(queue_key)

        if not exists:
            health_report["status"] = "error"
            health_report["issues"].append(f"队列不存在")
            return health_report

        # 2. 获取队列当前状态
        queue_length = await redis_client.xlen(queue_key)
        health_report["queue_length"] = queue_length

        # 3. 检查消费者组
        try:
            groups = await redis_client.xinfo_groups(queue_key)
            health_report["consumer_groups"] = len(groups)
            health_report["total_pending"] = sum(g.get('pending', 0) for g in groups)

            if len(groups) == 0:
                health_report["issues"].append("没有消费者组")
                health_report["recommendations"].append("创建消费者组以处理任务")
        except:
            health_report["consumer_groups"] = 0
            health_report["total_pending"] = 0
            health_report["issues"].append("无法获取消费者组信息")

        # 4. 从 PostgreSQL 获取历史失败率（如果配置了）
        if ns.pg_config:
            async with await ns.get_pg_session() as session:
                from sqlalchemy import text

                # 查询最近24小时的任务失败率
                query = text("""
                    SELECT
                        COUNT(*) FILTER (WHERE status = 'failed') as failed_count,
                        COUNT(*) as total_count
                    FROM task_runs
                    WHERE queue_name = :queue_name
                      AND created_at >= NOW() - INTERVAL '24 hours'
                """)

                result = await session.execute(query, {"queue_name": queue_name})
                row = result.fetchone()

                if row and row.total_count > 0:
                    failure_rate = (row.failed_count / row.total_count) * 100
                    health_report["failure_rate_24h"] = round(failure_rate, 2)

                    if failure_rate > 10:
                        health_report["issues"].append(f"失败率过高: {failure_rate:.2f}%")
                        health_report["recommendations"].append("检查任务执行逻辑和错误日志")

        # 5. 评估整体健康状态
        if len(health_report["issues"]) == 0:
            health_report["status"] = "healthy"
        elif len(health_report["issues"]) <= 2:
            health_report["status"] = "warning"
        else:
            health_report["status"] = "critical"

        # 6. 添加积压警告
        if queue_length > 1000:
            health_report["issues"].append(f"队列积压严重: {queue_length} 条消息")
            health_report["recommendations"].append("增加消费者数量或检查处理性能")

        return {"success": True, "health": health_report}

    finally:
        await redis_client.aclose()


# ============================================================================
# 对比总结
# ============================================================================

"""
重构前后对比总结:

1. 代码行数
   - 重构前: 平均 30-40 行/路由
   - 重构后: 平均 15-20 行/路由
   - 减少: 50-60%

2. 样板代码
   - 重构前: 每个路由重复 10+ 行样板代码
   - 重构后: 0 行样板代码
   - 减少: 100%

3. 错误处理
   - 重构前: 手动 try-catch，容易遗漏
   - 重构后: 统一处理，不会遗漏
   - 改善: 显著提高

4. 可维护性
   - 重构前: 修改需要改多处
   - 重构后: 集中修改一处
   - 改善: 显著提高

5. 类型安全
   - 重构前: 基本没有类型提示
   - 重构后: 完整的类型提示
   - 改善: 显著提高

6. 开发体验
   - 重构前: IDE 无法提供代码补全
   - 重构后: 完整的代码补全和类型检查
   - 改善: 显著提高
"""

__all__ = ['router']



"""
时间处理工具函数
"""
import json
from datetime import datetime, timezone
from typing import Dict, Any, List


def parse_iso_datetime(time_str: str) -> datetime:
    """
    解析 ISO 格式的时间字符串，确保返回 UTC 时间

    Args:
        time_str: ISO 格式的时间字符串

    Returns:
        UTC 时间的 datetime 对象
    """
    if time_str.endswith('Z'):
        # Z 表示 UTC 时间
        dt = datetime.fromisoformat(time_str.replace('Z', '+00:00'))
    else:
        dt = datetime.fromisoformat(time_str)

    # 如果没有时区信息，假定为 UTC
    if dt.tzinfo is None:
        dt = dt.replace(tzinfo=timezone.utc)
    # 如果有时区信息，转换为 UTC
    elif dt.tzinfo != timezone.utc:
        dt = dt.astimezone(timezone.utc)

    return dt


def format_task_timestamps(task: Dict[str, Any], fields: List[str] = None) -> Dict[str, Any]:
    """
    将任务对象中的时间字段转换为 ISO 格式字符串

    Args:
        task: 任务字典
        fields: 需要转换的字段列表，默认为 ['created_at', 'started_at', 'completed_at']

    Returns:
        转换后的任务字典
    """
    if fields is None:
        fields = ['created_at', 'started_at', 'completed_at']

    for field in fields:
        if task.get(field):
            # PostgreSQL 的 TIMESTAMP WITH TIME ZONE 会返回 aware datetime
            if task[field].tzinfo is None:
                # 如果没有时区信息，假定为 UTC
                task[field] = task[field].replace(tzinfo=timezone.utc)
            task[field] = task[field].isoformat()

    return task


def parse_task_json_fields(task: Dict[str, Any], fields: List[str] = None) -> Dict[str, Any]:
    """
    解析任务对象中的 JSON 字段

    Args:
        task: 任务字典
        fields: 需要解析的字段列表，默认为 ['task_data', 'result', 'metadata']

    Returns:
        解析后的任务字典
    """
    if fields is None:
        fields = ['task_data', 'result', 'metadata']

    for field in fields:
        if task.get(field) and isinstance(task[field], str):
            try:
                task[field] = json.loads(task[field])
            except:
                pass

    return task


def task_obj_to_dict(task_obj) -> Dict[str, Any]:
    """
    将 Task ORM 对象转换为字典

    Args:
        task_obj: Task ORM 对象

    Returns:
        任务字典
    """
    task = {
        'id': task_obj.id,
        'queue_name': task_obj.queue_name,
        'task_name': task_obj.task_name,
        'task_data': task_obj.task_data,
        'priority': task_obj.priority,
        'retry_count': task_obj.retry_count,
        'max_retry': task_obj.max_retry,
        'status': task_obj.status,
        'result': task_obj.result,
        'error_message': task_obj.error_message,
        'created_at': task_obj.created_at,
        'started_at': task_obj.started_at,
        'completed_at': task_obj.completed_at,
        'worker_id': task_obj.worker_id,
        'execution_time': task_obj.execution_time,
        'duration': task_obj.duration,
        'metadata': task_obj.task_metadata,
        'next_sync_time': task_obj.next_sync_time,
        'sync_check_count': task_obj.sync_check_count
    }

    # 转换时间戳为 ISO 格式
    task = format_task_timestamps(task)

    # 解析 JSON 字段
    task = parse_task_json_fields(task)

    return task



"""
任务加载器 - 负责从数据库加载任务到Redis
"""
import asyncio
from redis import asyncio as aioredis
from typing import Optional, Set
from datetime import datetime, timedelta
import json

from ..utils.task_logger import get_task_logger, LogContext
from .database import ScheduledTaskManager
from .models import ScheduledTask


logger = get_task_logger(__name__)


class TaskLoader:
    """
    任务加载器
    定期从数据库加载即将执行的任务到Redis ZSET
    """
    
    def __init__(
        self,
        redis_url: str,
        db_manager: ScheduledTaskManager,
        redis_prefix: str = "jettask:scheduled",
        lookahead_minutes: int = 5,
        load_interval: int = 30,
        sync_interval_cycles: int = 2  # 每几个周期同步一次
    ):
        """
        初始化加载器
        
        Args:
            redis_url: Redis连接URL
            db_manager: 数据库管理器
            redis_prefix: Redis键前缀
            lookahead_minutes: 向前查看的分钟数（加载未来N分钟的任务）
            load_interval: 加载间隔（秒）
            sync_interval_cycles: 每几个周期与数据库同步一次（默认2个周期=1分钟）
        """
        self.sync_interval_cycles = sync_interval_cycles
        self.redis_url = redis_url
        self.db_manager = db_manager
        self.redis_prefix = redis_prefix
        self.lookahead_minutes = lookahead_minutes
        self.load_interval = load_interval
        
        self.redis: Optional[aioredis.Redis] = None
        self.running = False
        self.loaded_tasks: Set[str] = set()  # 已加载的任务ID
    
    async def connect(self):
        """建立Redis连接（使用统一的连接池管理）"""
        if not self.redis:
            from jettask.db.connector import get_async_redis_client

            self.redis = get_async_redis_client(
                redis_url=self.redis_url,
                decode_responses=False
            )
    
    async def disconnect(self):
        """关闭Redis连接"""
        if self.redis:
            await self.redis.close()
            self.redis = None
    
    def _get_zset_key(self) -> str:
        """获取ZSET键名"""
        return f"{self.redis_prefix}:tasks"
    
    def _get_task_detail_key(self, task_id: int) -> str:
        """获取任务详情键名"""
        return f"{self.redis_prefix}:task:{task_id}"
    
    async def load_tasks(self) -> int:
        """
        从数据库加载任务到Redis
        
        Returns:
            加载的任务数量
        """
        with LogContext(operation="load_tasks"):
            try:
                # 获取即将执行的任务
                tasks = await self.db_manager.get_ready_tasks(
                    batch_size=1000,
                    lookahead_seconds=self.lookahead_minutes * 60
                )
                
                if not tasks:
                    logger.debug("No tasks to load")
                    return 0
                
                # 批量添加到Redis
                pipe = self.redis.pipeline()
                loaded_count = 0
                
                for task in tasks:
                    if not task.enabled or not task.next_run_time:
                        continue
                    
                    task_id = task.id
                    score = task.next_run_time.timestamp()
                    
                    # 添加到ZSET（用于调度）
                    pipe.zadd(self._get_zset_key(), {str(task_id): score})  # Redis ZSET 需要字符串键
                    
                    # 存储任务详情
                    pipe.setex(
                        self._get_task_detail_key(task_id),
                        self.lookahead_minutes * 60 + 60,  # 过期时间比lookahead稍长
                        task.to_redis_value()
                    )
                    
                    self.loaded_tasks.add(task_id)
                    loaded_count += 1
                
                await pipe.execute()
                
                logger.info(f"Loaded {loaded_count} tasks to Redis", 
                           extra={'extra_fields': {'task_count': loaded_count}})
                
                return loaded_count
                
            except Exception as e:
                logger.error(f"Failed to load tasks: {e}", exc_info=True)
                raise
    
    async def remove_task(self, task_id: int):
        """
        从Redis中移除任务
        
        Args:
            task_id: 任务ID
        """
        pipe = self.redis.pipeline()
        pipe.zrem(self._get_zset_key(), str(task_id))
        pipe.delete(self._get_task_detail_key(task_id))
        await pipe.execute()
        
        self.loaded_tasks.discard(task_id)
        
        logger.info(f"Removed task {task_id} from Redis")
    
    async def update_task_score(self, task_id: int, next_run_time: datetime):
        """
        更新任务在ZSET中的分数（下次执行时间）
        
        Args:
            task_id: 任务ID
            next_run_time: 下次执行时间
        """
        score = next_run_time.timestamp()
        await self.redis.zadd(self._get_zset_key(), {str(task_id): score})
        
        logger.debug(f"Updated task {task_id} score to {score}")
    
    async def cleanup_expired(self):
        """清理过期的任务"""
        # 移除已过期很久的任务（比如1小时前的）
        cutoff_time = datetime.now() - timedelta(hours=1)
        cutoff_score = cutoff_time.timestamp()
        
        removed = await self.redis.zremrangebyscore(
            self._get_zset_key(),
            '-inf',
            cutoff_score
        )
        
        if removed:
            logger.info(f"Cleaned up {removed} expired tasks from Redis")
    
    async def sync_with_db(self):
        """
        与数据库同步任务状态
        检查Redis中的任务是否仍然有效
        """
        # 获取Redis中所有任务ID
        redis_tasks = await self.redis.zrange(self._get_zset_key(), 0, -1)
        redis_task_ids = {int(task_id.decode() if isinstance(task_id, bytes) else task_id) 
                         for task_id in redis_tasks}  # 转换为整数
        
        # 批量检查这些任务在数据库中的状态
        for task_id in redis_task_ids:
            db_task = await self.db_manager.get_task(task_id)  # task_id 现在是整数
            
            if not db_task or not db_task.enabled:
                # 任务已删除或禁用，从Redis移除
                await self.remove_task(task_id)
            elif db_task.next_run_time:
                # 更新执行时间
                await self.update_task_score(task_id, db_task.next_run_time)
    
    async def run(self):
        """运行加载器主循环"""
        self.running = True
        logger.info("Task loader started")
        
        # 主循环
        cycle_count = 0
        first_run = True
        
        while self.running:
            try:
                # 确定本次循环的类型
                if first_run:
                    cycle_type = "initial_load"
                    logger.info("Performing initial task load...")
                else:
                    cycle_type = "load"
                
                with LogContext(cycle=cycle_type):
                    # 加载任务
                    await self.load_tasks()
                    
                    # 初始加载时执行同步
                    if first_run:
                        await self.sync_with_db()
                        first_run = False
                    else:
                        cycle_count += 1
                        
                        # 每2个周期同步一次数据库（默认1分钟）
                        if cycle_count % self.sync_interval_cycles == 0:
                            await self.sync_with_db()
                        
                        # 每5个周期清理一次过期任务
                        if cycle_count % 5 == 0:
                            await self.cleanup_expired()
                        
                        # 重置计数器，避免溢出
                        if cycle_count >= 100:
                            cycle_count = 0
                
            except Exception as e:
                logger.error(f"Loader cycle error: {e}", exc_info=True)
            
            # 等待间隔（放在循环末尾）
            if self.running:  # 只有在继续运行时才等待
                await asyncio.sleep(self.load_interval)
        
        logger.info("Task loader stopped")
    
    def stop(self):
        """停止加载器"""
        self.running = False


"""
数据库模型定义

所有表的 SQLAlchemy 模型定义
"""

from .task import Task
from .task_run import TaskRun
from .scheduled_task import ScheduledTask, TaskExecutionHistory

__all__ = [
    'Task',
    'TaskRun',
    'ScheduledTask',
    'TaskExecutionHistory',
]



"""
定时任务服务层
处理定时任务相关的业务逻辑
"""
from typing import Optional, List, Dict, Any
import logging

logger = logging.getLogger(__name__)


class ScheduledTaskService:
    """定时任务服务类"""
    
    def __init__(self, data_access):
        """
        初始化定时任务服务
        
        Args:
            data_access: 数据访问层实例
        """
        self.data_access = data_access
    
    @staticmethod
    def validate_schedule_config(schedule_type: str, schedule_config: dict):
        """
        验证调度配置
        
        Args:
            schedule_type: 调度类型
            schedule_config: 调度配置
            
        Raises:
            ValueError: 配置无效时
        """
        if schedule_type == 'interval':
            if 'seconds' in schedule_config:
                seconds = schedule_config.get('seconds')
                if seconds is None or seconds <= 0:
                    raise ValueError(f"间隔时间必须大于0秒，当前值: {seconds}")
                if seconds < 1:
                    raise ValueError(
                        f"间隔时间不能小于1秒，当前值: {seconds}秒。"
                        f"小于1秒的高频任务可能影响系统性能"
                    )
            elif 'minutes' in schedule_config:
                minutes = schedule_config.get('minutes')
                if minutes is None or minutes <= 0:
                    raise ValueError(f"间隔时间必须大于0分钟，当前值: {minutes}")
            else:
                raise ValueError("interval类型的任务必须指定seconds或minutes")
        elif schedule_type == 'cron':
            if 'cron_expression' not in schedule_config:
                raise ValueError("cron类型的任务必须指定cron_expression")
    
    async def get_scheduled_tasks(
        self,
        page: int = 1,
        page_size: int = 20,
        search: Optional[str] = None,
        is_active: Optional[bool] = None,
        filters: Optional[List[Dict]] = None,
        time_range: Optional[str] = None,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        获取定时任务列表
        
        Args:
            page: 页码
            page_size: 每页大小
            search: 搜索关键字
            is_active: 是否激活
            filters: 筛选条件
            time_range: 时间范围
            start_time: 开始时间
            end_time: 结束时间
            
        Returns:
            任务列表和总数
        """
        async with self.data_access.get_session() as session:
            tasks, total = await self.data_access.fetch_scheduled_tasks(
                session=session,
                page=page,
                page_size=page_size,
                search=search,
                is_active=is_active,
                filters=filters,
                time_range=time_range,
                start_time=start_time,
                end_time=end_time
            )
        
        return {
            "success": True,
            "data": tasks,
            "total": total,
            "page": page,
            "page_size": page_size
        }
    
    async def create_scheduled_task(
        self,
        task_data: Dict[str, Any]
    ) -> Dict[str, Any]:
        """
        创建定时任务
        
        Args:
            task_data: 任务数据
            
        Returns:
            创建的任务信息
        """
        # 验证调度配置
        self.validate_schedule_config(
            task_data['schedule_type'],
            task_data['schedule_config']
        )
        
        async with self.data_access.get_session() as session:
            task = await self.data_access.create_scheduled_task(session, task_data)
        
        return {
            "success": True,
            "data": task,
            "message": "定时任务创建成功"
        }
    
    async def update_scheduled_task(
        self,
        task_id: str,
        task_data: Dict[str, Any]
    ) -> Dict[str, Any]:
        """
        更新定时任务
        
        Args:
            task_id: 任务ID
            task_data: 任务数据
            
        Returns:
            更新后的任务信息
        """
        # 验证调度配置
        self.validate_schedule_config(
            task_data['schedule_type'],
            task_data['schedule_config']
        )
        
        async with self.data_access.get_session() as session:
            task = await self.data_access.update_scheduled_task(
                session, task_id, task_data
            )
        
        return {
            "success": True,
            "data": task,
            "message": "定时任务更新成功"
        }
    
    async def delete_scheduled_task(self, task_id: str) -> Dict[str, Any]:
        """
        删除定时任务
        
        Args:
            task_id: 任务ID
            
        Returns:
            操作结果
        """
        async with self.data_access.get_session() as session:
            success = await self.data_access.delete_scheduled_task(session, task_id)
        
        if success:
            return {
                "success": True,
                "message": f"定时任务 {task_id} 已删除"
            }
        else:
            raise ValueError("定时任务不存在")
    
    async def toggle_scheduled_task(self, task_id: str) -> Dict[str, Any]:
        """
        启用/禁用定时任务
        
        Args:
            task_id: 任务ID
            
        Returns:
            更新后的状态
        """
        async with self.data_access.get_session() as session:
            task = await self.data_access.toggle_scheduled_task(session, task_id)
        
        if task:
            return {
                "success": True,
                "data": {
                    "id": task["id"],
                    "is_active": task["enabled"]
                },
                "message": "定时任务状态已更新"
            }
        else:
            raise ValueError("定时任务不存在")


"""
执行器模块

提供任务执行的核心功能，支持单进程和多进程两种模式。

主要组件：
- TaskExecutor: 任务执行器（新）
- ProcessOrchestrator: 多进程编排器
- ExecutorCore: 核心执行逻辑
"""

from .core import ExecutorCore
from .orchestrator import ProcessConfig, ProcessOrchestrator
from .task_executor import TaskExecutor


__all__ = [
    # 新的推荐类
    'TaskExecutor',
    'ProcessOrchestrator',
    'ProcessConfig',
    'ExecutorCore',
]



"""
限流器 - 兼容性导入

⚠️ 此文件保持向后兼容，将导入重定向到新的模块化结构。
原代码已被拆分到以下模块：
- jettask.rate_limit.qps_limiter: QPSRateLimiter 类
- jettask.rate_limit.concurrency_limiter: ConcurrencyRateLimiter 类
- jettask.rate_limit.task_limiter: TaskRateLimiter 类
- jettask.rate_limit.manager: RateLimiterManager 类

新的模块结构：
- qps_limiter.py: QPS限流器
- concurrency_limiter.py: 并发限流器
- task_limiter.py: 任务级限流器
- manager.py: 限流器管理器

使用示例：
    # 旧的导入方式（仍然可用）
    from jettask.rate_limit.limiter import RateLimiterManager, QPSRateLimiter

    # 新的导入方式（推荐）
    from jettask.rate_limit import RateLimiterManager, QPSRateLimiter
"""

# 导入新模块的所有公开接口
from .config import RateLimitConfig, QPSLimit, ConcurrencyLimit
from .qps_limiter import QPSRateLimiter
from .concurrency_limiter import ConcurrencyRateLimiter
from .task_limiter import TaskRateLimiter
from .manager import RateLimiterManager

# 保持向后兼容
__all__ = [
    'RateLimitConfig',
    'QPSLimit',
    'ConcurrencyLimit',
    'QPSRateLimiter',
    'ConcurrencyRateLimiter',
    'TaskRateLimiter',
    'RateLimiterManager',
]



"""
限流器管理器

统一管理所有任务的限流器实例
职责：
1. 限流器生命周期管理
2. 配置动态更新
3. 统一的限流接口
"""

import asyncio
import logging
import time
from redis.asyncio import Redis
from typing import Dict, Optional, List, Any

from .task_limiter import TaskRateLimiter
from .config import RateLimitConfig, parse_rate_limit_config

logger = logging.getLogger('app')


class RateLimiterManager:
    """限流器管理器

    管理多个任务的限流器。
    """

    def __init__(
        self,
        redis_client: Redis,
        worker_id: str,
        redis_prefix: str = "jettask",
        worker_state_manager = None
    ):
        """初始化限流器管理器

        Args:
            redis_client: 异步 Redis 客户端
            worker_id: 当前 worker ID
            redis_prefix: Redis key 前缀
            worker_state_manager: WorkerManager 实例
        """
        self.redis = redis_client
        self.worker_id = worker_id
        self.redis_prefix = redis_prefix
        self.worker_state_manager = worker_state_manager

        # 任务名 -> 限流器映射
        self.limiters: Dict[str, TaskRateLimiter] = {}

        logger.debug(f"RateLimiterManager initialized for worker {worker_id}")

    @staticmethod
    def register_rate_limit_config(redis_client, task_name: str, config: RateLimitConfig, redis_prefix: str = "jettask"):
        """注册任务的限流配置到 Redis（同步方法）

        Args:
            redis_client: 同步 Redis 客户端
            task_name: 任务名称
            config: RateLimitConfig 对象（ConcurrencyLimit 或 QPSLimit）
            redis_prefix: Redis key 前缀
        """
        try:
            rate_limit_key = f"{redis_prefix}:RATE_LIMIT:CONFIG:{task_name}"
            # 将配置对象转换为字典并保存到 Redis
            config_dict = config.to_dict()
            redis_client.hset(rate_limit_key, mapping=config_dict)

            # 将任务名添加到索引集合中（用于避免 scan 操作）
            index_key = f"{redis_prefix}:RATE_LIMIT:INDEX"
            redis_client.sadd(index_key, task_name)

            logger.debug(f"Registered rate limit config for task '{task_name}': {config}")
        except Exception as e:
            logger.error(f"Failed to register rate limit config for task '{task_name}': {e}")

    @staticmethod
    def unregister_rate_limit_config(redis_client, task_name: str, redis_prefix: str = "jettask"):
        """从 Redis 中删除任务的限流配置（同步方法）

        Args:
            redis_client: 同步 Redis 客户端
            task_name: 任务名称
            redis_prefix: Redis key 前缀

        Returns:
            是否成功删除
        """
        try:
            rate_limit_key = f"{redis_prefix}:RATE_LIMIT:CONFIG:{task_name}"
            deleted = redis_client.delete(rate_limit_key)

            if deleted:
                # 从索引集合中移除
                index_key = f"{redis_prefix}:RATE_LIMIT:INDEX"
                redis_client.srem(index_key, task_name)
                logger.debug(f"Removed rate limit config for task '{task_name}'")
                return True
            return False
        except Exception as e:
            logger.error(f"Failed to remove rate limit config for task '{task_name}': {e}")
            return False

    async def add_limiter(self, task_name: str, config: RateLimitConfig):
        """添加限流器

        Args:
            task_name: 任务名称
            config: 限流配置（QPSLimit 或 ConcurrencyLimit）
        """
        if task_name in self.limiters:
            logger.warning(f"Limiter for {task_name} already exists")
            return

        limiter = TaskRateLimiter(
            redis_client=self.redis,
            task_name=task_name,
            worker_id=self.worker_id,
            config=config,
            redis_prefix=self.redis_prefix,
            worker_state_manager=self.worker_state_manager
        )

        await limiter.start()
        self.limiters[task_name] = limiter

        logger.debug(f"Added rate limiter for {task_name}: {config}")

    async def remove_limiter(self, task_name: str, remove_from_redis: bool = False):
        """移除任务限流器

        Args:
            task_name: 任务名称
            remove_from_redis: 是否从 Redis 中删除配置（默认 False）
        """
        if task_name not in self.limiters:
            return

        limiter = self.limiters.pop(task_name)
        await limiter.stop()

        # 如果需要，从 Redis 中删除配置和索引
        if remove_from_redis:
            try:
                # 删除配置 Hash
                rate_limit_key = f"{self.redis_prefix}:RATE_LIMIT:CONFIG:{task_name}"
                await self.redis.delete(rate_limit_key)

                # 从索引集合中移除
                index_key = f"{self.redis_prefix}:RATE_LIMIT:INDEX"
                await self.redis.srem(index_key, task_name)

                logger.debug(f"Removed rate limit config and index for {task_name} from Redis")
            except Exception as e:
                logger.error(f"Failed to remove rate limit config from Redis: {e}")

        logger.debug(f"Removed rate limiter for {task_name}")

    async def load_config_from_redis(self, task_names: list = None):
        """从 Redis 加载限流配置

        Args:
            task_names: 任务名称列表（如果提供，只加载这些任务的配置；否则从索引集合中加载）
        """
        try:
            config_count = 0
            loaded_limiters = []

            # 如果没有提供 task_names，尝试从索引集合中获取
            if not task_names:
                index_key = f"{self.redis_prefix}:RATE_LIMIT:INDEX"
                task_names_bytes = await self.redis.smembers(index_key)
                task_names = [
                    name.decode('utf-8') if isinstance(name, bytes) else name
                    for name in task_names_bytes
                ]

                if not task_names:
                    logger.debug(f"No rate limit configs found in index {index_key}")
                    return

            # 遍历所有任务名称，加载配置
            for task_name in task_names:
                key = f"{self.redis_prefix}:RATE_LIMIT:CONFIG:{task_name}"

                # 检查 key 是否存在
                exists = await self.redis.exists(key)
                if not exists:
                    continue

                # 检查 key 类型
                key_type = await self.redis.type(key)
                if key_type != "hash":
                    logger.debug(f"Skipping non-hash key: {key} (type: {key_type})")
                    continue

                # 从 Hash 中读取配置
                config_dict = await self.redis.hgetall(key)
                if not config_dict:
                    continue

                # 转换 bytes 为 str
                config_dict = {
                    k.decode() if isinstance(k, bytes) else k:
                    v.decode() if isinstance(v, bytes) else v
                    for k, v in config_dict.items()
                }

                # 解析配置
                config = parse_rate_limit_config(config_dict)
                if config and task_name not in self.limiters:
                    try:
                        await self.add_limiter(task_name, config)
                        config_count += 1
                        loaded_limiters.append(task_name)
                    except Exception as e:
                        logger.error(f"Failed to add limiter for {task_name}: {e}")

            logger.debug(f"Loaded {config_count} rate limit configs from Redis")
            logger.debug(f"Loaded rate limit config from Redis, limiters: {loaded_limiters}")
        except Exception as e:
            logger.error(f"Failed to load config from Redis: {e}")

    async def acquire(self, task_name: str, timeout: float = 10.0) -> Optional[str]:
        """获取指定任务的执行许可

        Returns:
            成功返回 task_id (或 True), 失败返回 None
        """
        limiter = self.limiters.get(task_name)
        if not limiter:
            # 没有限流，直接返回 True
            return True

        return await limiter.acquire(timeout)

    async def release(self, task_name: str, task_id: Optional[str] = None):
        """释放指定任务的执行许可

        Args:
            task_name: 任务名称
            task_id: 任务ID (ConcurrencyLimit 需要)
        """
        limiter = self.limiters.get(task_name)
        if limiter:
            await limiter.release(task_id)

    async def stop_all(self):
        """停止所有限流器"""
        for task_name, limiter in list(self.limiters.items()):
            await limiter.stop()
        self.limiters.clear()
        logger.debug("Stopped all rate limiters")

    def get_all_stats(self) -> Dict[str, dict]:
        """获取所有限流器的统计信息"""
        stats = {}
        for task_name, limiter in self.limiters.items():
            if hasattr(limiter, 'get_stats'):
                stats[task_name] = limiter.get_stats()
        return stats


__all__ = ['RateLimiterManager']



from .helpers import get_hostname, gen_task_name, is_async_function
from .file_watcher import FileChangeHandler
from .task_logger import get_task_logger

# 为了向后兼容，从 jettask.db 重新导出数据库连接函数
from jettask.db.connector import (
    # 全局连接池函数
    get_sync_redis_pool,
    get_async_redis_pool,
    get_pg_engine_and_factory,
    # 客户端实例函数
    get_sync_redis_client,
    get_async_redis_client,
    get_dual_mode_async_redis_client,
    # 配置解析
    DBConfig,
)

__all__ = [
    "get_hostname",
    "gen_task_name",
    "is_async_function",
    # 全局连接池函数（向后兼容）
    "get_sync_redis_pool",
    "get_async_redis_pool",
    "get_pg_engine_and_factory",
    # 客户端实例函数
    "get_sync_redis_client",
    "get_async_redis_client",
    "get_dual_mode_async_redis_client",
    # 数据库连接工具
    "DBConfig",
    "FileChangeHandler",
    "get_task_logger",
]